# WARNING: This file is generated from the API header files
# It is used to set the appropriate return and arguments types of the exposed functions 
# from the MetaWear shared library so Python can correctly call into the library
from ctypes import *
from .core import *
from .peripheral import Led
from .sensor import SPI

def setup_libmetawear(libmetawear):
    libmetawear.mbl_mw_memory_free.restype= None
    libmetawear.mbl_mw_memory_free.argtypes= [c_void_p]

    libmetawear.mbl_mw_debug_reset.restype= None
    libmetawear.mbl_mw_debug_reset.argtypes= [c_void_p]

    libmetawear.mbl_mw_debug_jump_to_bootloader.restype= None
    libmetawear.mbl_mw_debug_jump_to_bootloader.argtypes= [c_void_p]

    libmetawear.mbl_mw_debug_disconnect.restype= None
    libmetawear.mbl_mw_debug_disconnect.argtypes= [c_void_p]

    libmetawear.mbl_mw_debug_reset_after_gc.restype= None
    libmetawear.mbl_mw_debug_reset_after_gc.argtypes= [c_void_p]

    libmetawear.mbl_mw_event_get_owner.restype= c_void_p
    libmetawear.mbl_mw_event_get_owner.argtypes= [c_void_p]

    libmetawear.mbl_mw_event_record_commands.restype= None
    libmetawear.mbl_mw_event_record_commands.argtypes= [c_void_p]

    libmetawear.mbl_mw_event_end_record.restype= None
    libmetawear.mbl_mw_event_end_record.argtypes= [c_void_p, Fn_VoidPtr_Int]

    libmetawear.mbl_mw_logging_start.restype= None
    libmetawear.mbl_mw_logging_start.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_logging_stop.restype= None
    libmetawear.mbl_mw_logging_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_logging_clear_entries.restype= None
    libmetawear.mbl_mw_logging_clear_entries.argtypes= [c_void_p]

    libmetawear.mbl_mw_logging_download.restype= None
    libmetawear.mbl_mw_logging_download.argtypes= [c_void_p, c_ubyte, POINTER(LogDownloadHandler)]

    libmetawear.mbl_mw_logger_get_id.restype= c_ubyte
    libmetawear.mbl_mw_logger_get_id.argtypes= [c_void_p]

    libmetawear.mbl_mw_logger_get_signal.restype= c_void_p
    libmetawear.mbl_mw_logger_get_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_logger_lookup_id.restype= c_void_p
    libmetawear.mbl_mw_logger_lookup_id.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_logger_remove.restype= None
    libmetawear.mbl_mw_logger_remove.argtypes= [c_void_p]

    libmetawear.mbl_mw_logger_subscribe.restype= None
    libmetawear.mbl_mw_logger_subscribe.argtypes= [c_void_p, Fn_DataPtr]

    libmetawear.mbl_mw_timer_create.restype= None
    libmetawear.mbl_mw_timer_create.argtypes= [c_void_p, c_uint, c_ushort, c_ubyte, Fn_VoidPtr]

    libmetawear.mbl_mw_timer_create_indefinite.restype= None
    libmetawear.mbl_mw_timer_create_indefinite.argtypes= [c_void_p, c_uint, c_ubyte, Fn_VoidPtr]

    libmetawear.mbl_mw_timer_get_id.restype= c_ubyte
    libmetawear.mbl_mw_timer_get_id.argtypes= [c_void_p]

    libmetawear.mbl_mw_timer_lookup_id.restype= c_void_p
    libmetawear.mbl_mw_timer_lookup_id.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_timer_start.restype= None
    libmetawear.mbl_mw_timer_start.argtypes= [c_void_p]

    libmetawear.mbl_mw_timer_stop.restype= None
    libmetawear.mbl_mw_timer_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_timer_remove.restype= None
    libmetawear.mbl_mw_timer_remove.argtypes= [c_void_p]

    libmetawear.mbl_mw_settings_get_disconnect_event.restype= c_void_p
    libmetawear.mbl_mw_settings_get_disconnect_event.argtypes= [c_void_p]

    libmetawear.mbl_mw_settings_get_battery_state_data_signal.restype= c_void_p
    libmetawear.mbl_mw_settings_get_battery_state_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_settings_set_device_name.restype= None
    libmetawear.mbl_mw_settings_set_device_name.argtypes= [c_void_p, c_void_p, c_ubyte]

    libmetawear.mbl_mw_settings_set_ad_interval.restype= None
    libmetawear.mbl_mw_settings_set_ad_interval.argtypes= [c_void_p, c_ushort, c_ubyte]

    libmetawear.mbl_mw_settings_set_tx_power.restype= None
    libmetawear.mbl_mw_settings_set_tx_power.argtypes= [c_void_p, c_byte]

    libmetawear.mbl_mw_settings_start_advertising.restype= None
    libmetawear.mbl_mw_settings_start_advertising.argtypes= [c_void_p]

    libmetawear.mbl_mw_settings_set_scan_response.restype= None
    libmetawear.mbl_mw_settings_set_scan_response.argtypes= [c_void_p, c_void_p, c_ubyte]

    libmetawear.mbl_mw_settings_set_connection_parameters.restype= None
    libmetawear.mbl_mw_settings_set_connection_parameters.argtypes= [c_void_p, c_float, c_float, c_ushort, c_ushort]

    libmetawear.mbl_mw_datasignal_get_component.restype= c_void_p
    libmetawear.mbl_mw_datasignal_get_component.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_datasignal_subscribe.restype= None
    libmetawear.mbl_mw_datasignal_subscribe.argtypes= [c_void_p, Fn_DataPtr]

    libmetawear.mbl_mw_datasignal_unsubscribe.restype= None
    libmetawear.mbl_mw_datasignal_unsubscribe.argtypes= [c_void_p]

    libmetawear.mbl_mw_datasignal_is_readable.restype= c_int
    libmetawear.mbl_mw_datasignal_is_readable.argtypes= [c_void_p]

    libmetawear.mbl_mw_datasignal_read.restype= None
    libmetawear.mbl_mw_datasignal_read.argtypes= [c_void_p]

    libmetawear.mbl_mw_datasignal_read_with_parameters.restype= None
    libmetawear.mbl_mw_datasignal_read_with_parameters.argtypes= [c_void_p, c_void_p]

    libmetawear.mbl_mw_datasignal_log.restype= None
    libmetawear.mbl_mw_datasignal_log.argtypes= [c_void_p, Fn_VoidPtr]

    libmetawear.mbl_mw_connection_notify_char_changed.restype= c_int
    libmetawear.mbl_mw_connection_notify_char_changed.argtypes= [c_void_p, c_void_p, c_ubyte]

    libmetawear.mbl_mw_metawearboard_notify_char_changed.restype= c_int
    libmetawear.mbl_mw_metawearboard_notify_char_changed.argtypes= [c_void_p, c_void_p, c_ubyte]

    libmetawear.mbl_mw_connection_char_read.restype= None
    libmetawear.mbl_mw_connection_char_read.argtypes= [c_void_p, c_void_p, c_void_p, c_ubyte]

    libmetawear.mbl_mw_metawearboard_char_read.restype= None
    libmetawear.mbl_mw_metawearboard_char_read.argtypes= [c_void_p, c_void_p, c_void_p, c_ubyte]

    libmetawear.mbl_mw_metawearboard_create.restype= c_void_p
    libmetawear.mbl_mw_metawearboard_create.argtypes= [POINTER(BtleConnection)]

    libmetawear.mbl_mw_metawearboard_free.restype= None
    libmetawear.mbl_mw_metawearboard_free.argtypes= [c_void_p]

    libmetawear.mbl_mw_metawearboard_set_time_for_response.restype= None
    libmetawear.mbl_mw_metawearboard_set_time_for_response.argtypes= [c_void_p, c_ushort]

    libmetawear.mbl_mw_metawearboard_initialize.restype= None
    libmetawear.mbl_mw_metawearboard_initialize.argtypes= [c_void_p, Fn_VoidPtr_Int]

    libmetawear.mbl_mw_metawearboard_tear_down.restype= None
    libmetawear.mbl_mw_metawearboard_tear_down.argtypes= [c_void_p]

    libmetawear.mbl_mw_metawearboard_is_initialized.restype= c_int
    libmetawear.mbl_mw_metawearboard_is_initialized.argtypes= [c_void_p]

    libmetawear.mbl_mw_metawearboard_lookup_module.restype= c_int
    libmetawear.mbl_mw_metawearboard_lookup_module.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_metawearboard_serialize.restype= c_void_p
    libmetawear.mbl_mw_metawearboard_serialize.argtypes= [c_void_p, POINTER(c_uint)]

    libmetawear.mbl_mw_metawearboard_deserialize.restype= c_int
    libmetawear.mbl_mw_metawearboard_deserialize.argtypes= [c_void_p, c_void_p, c_uint]

    libmetawear.mbl_mw_haptic_start_motor.restype= None
    libmetawear.mbl_mw_haptic_start_motor.argtypes= [c_void_p, c_float, c_ushort]

    libmetawear.mbl_mw_haptic_start_buzzer.restype= None
    libmetawear.mbl_mw_haptic_start_buzzer.argtypes= [c_void_p, c_ushort]

    libmetawear.mbl_mw_led_load_preset_pattern.restype= None
    libmetawear.mbl_mw_led_load_preset_pattern.argtypes= [POINTER(Led.Pattern), c_int]

    libmetawear.mbl_mw_led_write_pattern.restype= None
    libmetawear.mbl_mw_led_write_pattern.argtypes= [c_void_p, POINTER(Led.Pattern), c_int]

    libmetawear.mbl_mw_led_autoplay.restype= None
    libmetawear.mbl_mw_led_autoplay.argtypes= [c_void_p]

    libmetawear.mbl_mw_led_play.restype= None
    libmetawear.mbl_mw_led_play.argtypes= [c_void_p]

    libmetawear.mbl_mw_led_pause.restype= None
    libmetawear.mbl_mw_led_pause.argtypes= [c_void_p]

    libmetawear.mbl_mw_led_stop.restype= None
    libmetawear.mbl_mw_led_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_led_stop_and_clear.restype= None
    libmetawear.mbl_mw_led_stop_and_clear.argtypes= [c_void_p]

    libmetawear.mbl_mw_neopixel_init_slow_strand.restype= None
    libmetawear.mbl_mw_neopixel_init_slow_strand.argtypes= [c_void_p, c_ubyte, c_ubyte, c_ubyte, c_int]

    libmetawear.mbl_mw_neopixel_init_fast_strand.restype= None
    libmetawear.mbl_mw_neopixel_init_fast_strand.argtypes= [c_void_p, c_ubyte, c_ubyte, c_ubyte, c_int]

    libmetawear.mbl_mw_neopixel_free_strand.restype= None
    libmetawear.mbl_mw_neopixel_free_strand.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_neopixel_enable_hold.restype= None
    libmetawear.mbl_mw_neopixel_enable_hold.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_neopixel_disable_hold.restype= None
    libmetawear.mbl_mw_neopixel_disable_hold.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_neopixel_clear.restype= None
    libmetawear.mbl_mw_neopixel_clear.argtypes= [c_void_p, c_ubyte, c_ubyte, c_ubyte]

    libmetawear.mbl_mw_neopixel_set_color.restype= None
    libmetawear.mbl_mw_neopixel_set_color.argtypes= [c_void_p, c_ubyte, c_ubyte, c_ubyte, c_ubyte, c_ubyte]

    libmetawear.mbl_mw_neopixel_rotate.restype= None
    libmetawear.mbl_mw_neopixel_rotate.argtypes= [c_void_p, c_ubyte, c_ubyte, c_ushort, c_int]

    libmetawear.mbl_mw_neopixel_rotate_indefinitely.restype= None
    libmetawear.mbl_mw_neopixel_rotate_indefinitely.argtypes= [c_void_p, c_ubyte, c_ubyte, c_int]

    libmetawear.mbl_mw_neopixel_stop_rotation.restype= None
    libmetawear.mbl_mw_neopixel_stop_rotation.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_ibeacon_set_major.restype= None
    libmetawear.mbl_mw_ibeacon_set_major.argtypes= [c_void_p, c_ushort]

    libmetawear.mbl_mw_ibeacon_set_major_signal.restype= None
    libmetawear.mbl_mw_ibeacon_set_major_signal.argtypes= [c_void_p, c_void_p]

    libmetawear.mbl_mw_ibeacon_set_minor.restype= None
    libmetawear.mbl_mw_ibeacon_set_minor.argtypes= [c_void_p, c_ushort]

    libmetawear.mbl_mw_ibeacon_set_minor_signal.restype= None
    libmetawear.mbl_mw_ibeacon_set_minor_signal.argtypes= [c_void_p, c_void_p]

    libmetawear.mbl_mw_ibeacon_set_period.restype= None
    libmetawear.mbl_mw_ibeacon_set_period.argtypes= [c_void_p, c_ushort]

    libmetawear.mbl_mw_ibeacon_set_tx_power.restype= None
    libmetawear.mbl_mw_ibeacon_set_tx_power.argtypes= [c_void_p, c_byte]

    libmetawear.mbl_mw_ibeacon_set_rx_power.restype= None
    libmetawear.mbl_mw_ibeacon_set_rx_power.argtypes= [c_void_p, c_byte]

    libmetawear.mbl_mw_ibeacon_set_uuid.restype= None
    libmetawear.mbl_mw_ibeacon_set_uuid.argtypes= [c_void_p, c_void_p]

    libmetawear.mbl_mw_ibeacon_enable.restype= None
    libmetawear.mbl_mw_ibeacon_enable.argtypes= [c_void_p]

    libmetawear.mbl_mw_ibeacon_disable.restype= None
    libmetawear.mbl_mw_ibeacon_disable.argtypes= [c_void_p]

    libmetawear.mbl_mw_dataprocessor_accumulator_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_accumulator_create.argtypes= [c_void_p, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_accumulator_create_size.restype= c_int
    libmetawear.mbl_mw_dataprocessor_accumulator_create_size.argtypes= [c_void_p, c_ubyte, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_set_accumulator_state.restype= c_int
    libmetawear.mbl_mw_dataprocessor_set_accumulator_state.argtypes= [c_void_p, c_float]

    libmetawear.mbl_mw_dataprocessor_average_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_average_create.argtypes= [c_void_p, c_ubyte, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_average_reset.restype= c_int
    libmetawear.mbl_mw_dataprocessor_average_reset.argtypes= [c_void_p]

    libmetawear.mbl_mw_dataprocessor_average_modify_size.restype= c_int
    libmetawear.mbl_mw_dataprocessor_average_modify_size.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_dataprocessor_buffer_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_buffer_create.argtypes= [c_void_p, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_comparator_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_comparator_create.argtypes= [c_void_p, c_int, c_float, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_comparator_create_signed.restype= c_int
    libmetawear.mbl_mw_dataprocessor_comparator_create_signed.argtypes= [c_void_p, c_int, c_float, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_comparator_create_unsigned.restype= c_int
    libmetawear.mbl_mw_dataprocessor_comparator_create_unsigned.argtypes= [c_void_p, c_int, c_float, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_comparator_modify.restype= c_int
    libmetawear.mbl_mw_dataprocessor_comparator_modify.argtypes= [c_void_p, c_int, c_float]

    libmetawear.mbl_mw_dataprocessor_comparator_modify_signal.restype= c_int
    libmetawear.mbl_mw_dataprocessor_comparator_modify_signal.argtypes= [c_void_p, c_int, c_void_p]

    libmetawear.mbl_mw_dataprocessor_multi_comparator_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_multi_comparator_create.argtypes= [c_void_p, c_int, c_int, c_void_p, c_ubyte, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_multi_comparator_create_signed.restype= c_int
    libmetawear.mbl_mw_dataprocessor_multi_comparator_create_signed.argtypes= [c_void_p, c_int, c_int, c_void_p, c_ubyte, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_multi_comparator_create_unsigned.restype= c_int
    libmetawear.mbl_mw_dataprocessor_multi_comparator_create_unsigned.argtypes= [c_void_p, c_int, c_int, c_void_p, c_ubyte, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_multi_comparator_modify.restype= c_int
    libmetawear.mbl_mw_dataprocessor_multi_comparator_modify.argtypes= [c_void_p, c_int, c_void_p, c_ubyte]

    libmetawear.mbl_mw_dataprocessor_counter_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_counter_create.argtypes= [c_void_p, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_counter_create_size.restype= c_int
    libmetawear.mbl_mw_dataprocessor_counter_create_size.argtypes= [c_void_p, c_ubyte, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_counter_set_state.restype= c_int
    libmetawear.mbl_mw_dataprocessor_counter_set_state.argtypes= [c_void_p, c_uint]

    libmetawear.mbl_mw_dataprocessor_get_state_data_signal.restype= c_void_p
    libmetawear.mbl_mw_dataprocessor_get_state_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_dataprocessor_remove.restype= None
    libmetawear.mbl_mw_dataprocessor_remove.argtypes= [c_void_p]

    libmetawear.mbl_mw_dataprocessor_get_id.restype= c_ubyte
    libmetawear.mbl_mw_dataprocessor_get_id.argtypes= [c_void_p]

    libmetawear.mbl_mw_dataprocessor_lookup_id.restype= c_void_p
    libmetawear.mbl_mw_dataprocessor_lookup_id.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_dataprocessor_delta_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_delta_create.argtypes= [c_void_p, c_int, c_float, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_delta_set_reference.restype= c_int
    libmetawear.mbl_mw_dataprocessor_delta_set_reference.argtypes= [c_void_p, c_float]

    libmetawear.mbl_mw_dataprocessor_delta_modify_magnitude.restype= c_int
    libmetawear.mbl_mw_dataprocessor_delta_modify_magnitude.argtypes= [c_void_p, c_float]

    libmetawear.mbl_mw_dataprocessor_math_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_math_create.argtypes= [c_void_p, c_int, c_float, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_math_create_signed.restype= c_int
    libmetawear.mbl_mw_dataprocessor_math_create_signed.argtypes= [c_void_p, c_int, c_float, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_math_create_unsigned.restype= c_int
    libmetawear.mbl_mw_dataprocessor_math_create_unsigned.argtypes= [c_void_p, c_int, c_float, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_math_modify_rhs.restype= c_int
    libmetawear.mbl_mw_dataprocessor_math_modify_rhs.argtypes= [c_void_p, c_float]

    libmetawear.mbl_mw_dataprocessor_math_modify_rhs_signal.restype= c_int
    libmetawear.mbl_mw_dataprocessor_math_modify_rhs_signal.argtypes= [c_void_p, c_void_p]

    libmetawear.mbl_mw_dataprocessor_passthrough_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_passthrough_create.argtypes= [c_void_p, c_int, c_ushort, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_passthrough_set_count.restype= c_int
    libmetawear.mbl_mw_dataprocessor_passthrough_set_count.argtypes= [c_void_p, c_ushort]

    libmetawear.mbl_mw_dataprocessor_passthrough_modify.restype= c_int
    libmetawear.mbl_mw_dataprocessor_passthrough_modify.argtypes= [c_void_p, c_int, c_ushort]

    libmetawear.mbl_mw_dataprocessor_pulse_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_pulse_create.argtypes= [c_void_p, c_int, c_float, c_ushort, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_pulse_modify.restype= c_int
    libmetawear.mbl_mw_dataprocessor_pulse_modify.argtypes= [c_void_p, c_float, c_ushort]

    libmetawear.mbl_mw_dataprocessor_rms_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_rms_create.argtypes= [c_void_p, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_rss_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_rss_create.argtypes= [c_void_p, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_sample_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_sample_create.argtypes= [c_void_p, c_ubyte, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_sample_modify_bin_size.restype= c_int
    libmetawear.mbl_mw_dataprocessor_sample_modify_bin_size.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_dataprocessor_threshold_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_threshold_create.argtypes= [c_void_p, c_int, c_float, c_float, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_threshold_modify_boundary.restype= c_int
    libmetawear.mbl_mw_dataprocessor_threshold_modify_boundary.argtypes= [c_void_p, c_float, c_float]

    libmetawear.mbl_mw_dataprocessor_time_create.restype= c_int
    libmetawear.mbl_mw_dataprocessor_time_create.argtypes= [c_void_p, c_int, c_uint, Fn_VoidPtr]

    libmetawear.mbl_mw_dataprocessor_time_modify_period.restype= c_int
    libmetawear.mbl_mw_dataprocessor_time_modify_period.argtypes= [c_void_p, c_uint]

    libmetawear.mbl_mw_acc_bosch_get_acceleration_data_signal.restype= c_void_p
    libmetawear.mbl_mw_acc_bosch_get_acceleration_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bosch_get_high_freq_acceleration_data_signal.restype= c_void_p
    libmetawear.mbl_mw_acc_bosch_get_high_freq_acceleration_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bmi160_get_step_counter_data_signal.restype= c_void_p
    libmetawear.mbl_mw_acc_bmi160_get_step_counter_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bmi160_get_step_detector_data_signal.restype= c_void_p
    libmetawear.mbl_mw_acc_bmi160_get_step_detector_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bmi160_set_odr.restype= None
    libmetawear.mbl_mw_acc_bmi160_set_odr.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_acc_bma255_set_odr.restype= None
    libmetawear.mbl_mw_acc_bma255_set_odr.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_acc_bosch_set_range.restype= None
    libmetawear.mbl_mw_acc_bosch_set_range.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_acc_bosch_write_acceleration_config.restype= None
    libmetawear.mbl_mw_acc_bosch_write_acceleration_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bmi160_set_step_counter_mode.restype= None
    libmetawear.mbl_mw_acc_bmi160_set_step_counter_mode.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_acc_bmi160_enable_step_counter.restype= None
    libmetawear.mbl_mw_acc_bmi160_enable_step_counter.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bmi160_disable_step_counter.restype= None
    libmetawear.mbl_mw_acc_bmi160_disable_step_counter.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bmi160_write_step_counter_config.restype= None
    libmetawear.mbl_mw_acc_bmi160_write_step_counter_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bmi160_reset_step_counter.restype= None
    libmetawear.mbl_mw_acc_bmi160_reset_step_counter.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bmi160_enable_step_detector.restype= None
    libmetawear.mbl_mw_acc_bmi160_enable_step_detector.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bmi160_disable_step_detector.restype= None
    libmetawear.mbl_mw_acc_bmi160_disable_step_detector.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bosch_start.restype= None
    libmetawear.mbl_mw_acc_bosch_start.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bosch_stop.restype= None
    libmetawear.mbl_mw_acc_bosch_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bosch_enable_acceleration_sampling.restype= None
    libmetawear.mbl_mw_acc_bosch_enable_acceleration_sampling.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_bosch_disable_acceleration_sampling.restype= None
    libmetawear.mbl_mw_acc_bosch_disable_acceleration_sampling.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_mma8452q_get_acceleration_data_signal.restype= c_void_p
    libmetawear.mbl_mw_acc_mma8452q_get_acceleration_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_mma8452q_get_high_freq_acceleration_data_signal.restype= c_void_p
    libmetawear.mbl_mw_acc_mma8452q_get_high_freq_acceleration_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_mma8452q_set_odr.restype= None
    libmetawear.mbl_mw_acc_mma8452q_set_odr.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_acc_mma8452q_set_range.restype= None
    libmetawear.mbl_mw_acc_mma8452q_set_range.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_acc_mma8452q_write_acceleration_config.restype= None
    libmetawear.mbl_mw_acc_mma8452q_write_acceleration_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_mma8452q_start.restype= None
    libmetawear.mbl_mw_acc_mma8452q_start.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_mma8452q_stop.restype= None
    libmetawear.mbl_mw_acc_mma8452q_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_mma8452q_enable_acceleration_sampling.restype= None
    libmetawear.mbl_mw_acc_mma8452q_enable_acceleration_sampling.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_mma8452q_disable_acceleration_sampling.restype= None
    libmetawear.mbl_mw_acc_mma8452q_disable_acceleration_sampling.argtypes= [c_void_p]

    libmetawear.mbl_mw_als_ltr329_get_illuminance_data_signal.restype= c_void_p
    libmetawear.mbl_mw_als_ltr329_get_illuminance_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_als_ltr329_set_gain.restype= None
    libmetawear.mbl_mw_als_ltr329_set_gain.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_als_ltr329_set_integration_time.restype= None
    libmetawear.mbl_mw_als_ltr329_set_integration_time.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_als_ltr329_set_measurement_rate.restype= None
    libmetawear.mbl_mw_als_ltr329_set_measurement_rate.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_als_ltr329_write_config.restype= None
    libmetawear.mbl_mw_als_ltr329_write_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_als_ltr329_start.restype= None
    libmetawear.mbl_mw_als_ltr329_start.argtypes= [c_void_p]

    libmetawear.mbl_mw_als_ltr329_stop.restype= None
    libmetawear.mbl_mw_als_ltr329_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_baro_bosch_get_pressure_data_signal.restype= c_void_p
    libmetawear.mbl_mw_baro_bosch_get_pressure_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_baro_bosch_get_pressure_read_data_signal.restype= c_void_p
    libmetawear.mbl_mw_baro_bosch_get_pressure_read_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_baro_bosch_get_altitude_data_signal.restype= c_void_p
    libmetawear.mbl_mw_baro_bosch_get_altitude_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_baro_bosch_set_oversampling.restype= None
    libmetawear.mbl_mw_baro_bosch_set_oversampling.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_baro_bosch_set_iir_filter.restype= None
    libmetawear.mbl_mw_baro_bosch_set_iir_filter.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_baro_bmp280_set_standby_time.restype= None
    libmetawear.mbl_mw_baro_bmp280_set_standby_time.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_baro_bme280_set_standby_time.restype= None
    libmetawear.mbl_mw_baro_bme280_set_standby_time.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_baro_bosch_set_standby_time.restype= c_float
    libmetawear.mbl_mw_baro_bosch_set_standby_time.argtypes= [c_void_p, c_float]

    libmetawear.mbl_mw_baro_bosch_write_config.restype= None
    libmetawear.mbl_mw_baro_bosch_write_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_baro_bosch_start.restype= None
    libmetawear.mbl_mw_baro_bosch_start.argtypes= [c_void_p]

    libmetawear.mbl_mw_baro_bosch_stop.restype= None
    libmetawear.mbl_mw_baro_bosch_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_gpio_get_analog_input_data_signal.restype= c_void_p
    libmetawear.mbl_mw_gpio_get_analog_input_data_signal.argtypes= [c_void_p, c_ubyte, c_int]

    libmetawear.mbl_mw_gpio_get_digital_input_data_signal.restype= c_void_p
    libmetawear.mbl_mw_gpio_get_digital_input_data_signal.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_gpio_get_pin_monitor_data_signal.restype= c_void_p
    libmetawear.mbl_mw_gpio_get_pin_monitor_data_signal.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_gpio_set_pull_mode.restype= None
    libmetawear.mbl_mw_gpio_set_pull_mode.argtypes= [c_void_p, c_ubyte, c_int]

    libmetawear.mbl_mw_gpio_set_digital_output.restype= None
    libmetawear.mbl_mw_gpio_set_digital_output.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_gpio_clear_digital_output.restype= None
    libmetawear.mbl_mw_gpio_clear_digital_output.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_gpio_set_pin_change_type.restype= None
    libmetawear.mbl_mw_gpio_set_pin_change_type.argtypes= [c_void_p, c_ubyte, c_int]

    libmetawear.mbl_mw_gpio_start_pin_monitoring.restype= None
    libmetawear.mbl_mw_gpio_start_pin_monitoring.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_gpio_stop_pin_monitoring.restype= None
    libmetawear.mbl_mw_gpio_stop_pin_monitoring.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_humidity_bme280_get_percentage_data_signal.restype= c_void_p
    libmetawear.mbl_mw_humidity_bme280_get_percentage_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_humidity_bme280_set_oversampling.restype= None
    libmetawear.mbl_mw_humidity_bme280_set_oversampling.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_i2c_get_data_signal.restype= c_void_p
    libmetawear.mbl_mw_i2c_get_data_signal.argtypes= [c_void_p, c_ubyte, c_ubyte]

    libmetawear.mbl_mw_i2c_write.restype= None
    libmetawear.mbl_mw_i2c_write.argtypes= [c_void_p, c_ubyte, c_ubyte, c_void_p, c_ubyte]

    libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal.restype= c_void_p
    libmetawear.mbl_mw_multi_chnl_temp_get_temperature_data_signal.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_multi_chnl_temp_configure_ext_thermistor.restype= None
    libmetawear.mbl_mw_multi_chnl_temp_configure_ext_thermistor.argtypes= [c_void_p, c_ubyte, c_ubyte, c_ubyte, c_ubyte]

    libmetawear.mbl_mw_multi_chnl_temp_get_source.restype= c_int
    libmetawear.mbl_mw_multi_chnl_temp_get_source.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_multi_chnl_temp_get_num_channels.restype= c_ubyte
    libmetawear.mbl_mw_multi_chnl_temp_get_num_channels.argtypes= [c_void_p]

    libmetawear.mbl_mw_proximity_tsl2671_get_adc_data_signal.restype= c_void_p
    libmetawear.mbl_mw_proximity_tsl2671_get_adc_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_proximity_tsl2671_set_integration_time.restype= None
    libmetawear.mbl_mw_proximity_tsl2671_set_integration_time.argtypes= [c_void_p, c_float]

    libmetawear.mbl_mw_proximity_tsl2671_set_n_pulses.restype= None
    libmetawear.mbl_mw_proximity_tsl2671_set_n_pulses.argtypes= [c_void_p, c_ubyte]

    libmetawear.mbl_mw_proximity_tsl2671_set_receiver_channel.restype= None
    libmetawear.mbl_mw_proximity_tsl2671_set_receiver_channel.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_proximity_tsl2671_set_transmitter_current.restype= None
    libmetawear.mbl_mw_proximity_tsl2671_set_transmitter_current.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_proximity_tsl2671_write_config.restype= None
    libmetawear.mbl_mw_proximity_tsl2671_write_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_spi_get_data_signal.restype= c_void_p
    libmetawear.mbl_mw_spi_get_data_signal.argtypes= [c_void_p, c_ubyte, c_ubyte]

    libmetawear.mbl_mw_spi_write.restype= None
    libmetawear.mbl_mw_spi_write.argtypes= [c_void_p, POINTER(SPI.Parameters)]

    libmetawear.mbl_mw_switch_get_state_data_signal.restype= c_void_p
    libmetawear.mbl_mw_switch_get_state_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_gyro_bmi160_get_rotation_data_signal.restype= c_void_p
    libmetawear.mbl_mw_gyro_bmi160_get_rotation_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_gyro_bmi160_get_high_freq_rotation_data_signal.restype= c_void_p
    libmetawear.mbl_mw_gyro_bmi160_get_high_freq_rotation_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_gyro_bmi160_set_odr.restype= None
    libmetawear.mbl_mw_gyro_bmi160_set_odr.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_gyro_bmi160_set_range.restype= None
    libmetawear.mbl_mw_gyro_bmi160_set_range.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_gyro_bmi160_write_config.restype= None
    libmetawear.mbl_mw_gyro_bmi160_write_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_gyro_bmi160_start.restype= None
    libmetawear.mbl_mw_gyro_bmi160_start.argtypes= [c_void_p]

    libmetawear.mbl_mw_gyro_bmi160_stop.restype= None
    libmetawear.mbl_mw_gyro_bmi160_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_gyro_bmi160_enable_rotation_sampling.restype= None
    libmetawear.mbl_mw_gyro_bmi160_enable_rotation_sampling.argtypes= [c_void_p]

    libmetawear.mbl_mw_gyro_bmi160_disable_rotation_sampling.restype= None
    libmetawear.mbl_mw_gyro_bmi160_disable_rotation_sampling.argtypes= [c_void_p]

    libmetawear.mbl_mw_cd_tcs34725_get_adc_data_signal.restype= c_void_p
    libmetawear.mbl_mw_cd_tcs34725_get_adc_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_cd_tcs34725_set_integration_time.restype= None
    libmetawear.mbl_mw_cd_tcs34725_set_integration_time.argtypes= [c_void_p, c_float]

    libmetawear.mbl_mw_cd_tcs34725_set_gain.restype= None
    libmetawear.mbl_mw_cd_tcs34725_set_gain.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_cd_tcs34725_enable_illuminator_led.restype= None
    libmetawear.mbl_mw_cd_tcs34725_enable_illuminator_led.argtypes= [c_void_p]

    libmetawear.mbl_mw_cd_tcs34725_disable_illuminator_led.restype= None
    libmetawear.mbl_mw_cd_tcs34725_disable_illuminator_led.argtypes= [c_void_p]

    libmetawear.mbl_mw_cd_tcs34725_write_config.restype= None
    libmetawear.mbl_mw_cd_tcs34725_write_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal.restype= c_void_p
    libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_mag_bmm150_configure.restype= None
    libmetawear.mbl_mw_mag_bmm150_configure.argtypes= [c_void_p, c_ushort, c_ushort, c_int]

    libmetawear.mbl_mw_mag_bmm150_set_preset.restype= None
    libmetawear.mbl_mw_mag_bmm150_set_preset.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_mag_bmm150_enable_b_field_sampling.restype= None
    libmetawear.mbl_mw_mag_bmm150_enable_b_field_sampling.argtypes= [c_void_p]

    libmetawear.mbl_mw_mag_bmm150_disable_b_field_sampling.restype= None
    libmetawear.mbl_mw_mag_bmm150_disable_b_field_sampling.argtypes= [c_void_p]

    libmetawear.mbl_mw_mag_bmm150_start.restype= None
    libmetawear.mbl_mw_mag_bmm150_start.argtypes= [c_void_p]

    libmetawear.mbl_mw_mag_bmm150_stop.restype= None
    libmetawear.mbl_mw_mag_bmm150_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_sensor_fusion_get_data_signal.restype= c_void_p
    libmetawear.mbl_mw_sensor_fusion_get_data_signal.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_sensor_fusion_set_mode.restype= None
    libmetawear.mbl_mw_sensor_fusion_set_mode.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_sensor_fusion_set_acc_range.restype= None
    libmetawear.mbl_mw_sensor_fusion_set_acc_range.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_sensor_fusion_set_gyro_range.restype= None
    libmetawear.mbl_mw_sensor_fusion_set_gyro_range.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_sensor_fusion_write_config.restype= None
    libmetawear.mbl_mw_sensor_fusion_write_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_sensor_fusion_enable_data.restype= None
    libmetawear.mbl_mw_sensor_fusion_enable_data.argtypes= [c_void_p, c_int]

    libmetawear.mbl_mw_sensor_fusion_clear_enabled_mask.restype= None
    libmetawear.mbl_mw_sensor_fusion_clear_enabled_mask.argtypes= [c_void_p]

    libmetawear.mbl_mw_sensor_fusion_start.restype= None
    libmetawear.mbl_mw_sensor_fusion_start.argtypes= [c_void_p]

    libmetawear.mbl_mw_sensor_fusion_stop.restype= None
    libmetawear.mbl_mw_sensor_fusion_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_get_acceleration_data_signal.restype= c_void_p
    libmetawear.mbl_mw_acc_get_acceleration_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_get_high_freq_acceleration_data_signal.restype= c_void_p
    libmetawear.mbl_mw_acc_get_high_freq_acceleration_data_signal.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_set_odr.restype= c_float
    libmetawear.mbl_mw_acc_set_odr.argtypes= [c_void_p, c_float]

    libmetawear.mbl_mw_acc_set_range.restype= c_float
    libmetawear.mbl_mw_acc_set_range.argtypes= [c_void_p, c_float]

    libmetawear.mbl_mw_acc_write_acceleration_config.restype= None
    libmetawear.mbl_mw_acc_write_acceleration_config.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_start.restype= None
    libmetawear.mbl_mw_acc_start.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_stop.restype= None
    libmetawear.mbl_mw_acc_stop.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_enable_acceleration_sampling.restype= None
    libmetawear.mbl_mw_acc_enable_acceleration_sampling.argtypes= [c_void_p]

    libmetawear.mbl_mw_acc_disable_acceleration_sampling.restype= None
    libmetawear.mbl_mw_acc_disable_acceleration_sampling.argtypes= [c_void_p]


class Math:
    OPERATION_ADD= 1
    OPERATION_MULTIPLY= 2
    OPERATION_DIVIDE= 3
    OPERATION_MODULUS= 4
    OPERATION_EXPONENT= 5
    OPERATION_SQRT= 6
    OPERATION_LSHIFT= 7
    OPERATION_RSHIFT= 8
    OPERATION_SUBTRACT= 9
    OPERATION_ABS_VALUE= 10
    OPERATION_CONSTANT= 11

class Comparator:
    OPERATION_EQ = 0
    OPERATION_NEQ = 1
    OPERATION_LT = 2
    OPERATION_LTE = 3
    OPERATION_GT = 4
    OPERATION_GTE = 5

    MODE_ABSOLUTE = 0
    MODE_REFERENCE = 1
    MODE_ZONE = 2
    MODE_BINARY = 3

class Threshold:
    MODE_ABSOLUTE= 0
    MODE_BINARY= 1

class Time:
    MODE_ABSOLUTE= 0
    MODE_DIFFERENTIAL= 1

class Passthrough:
    MODE_ALL= 0
    MODE_CONDITIONAL= 1
    MODE_COUNT= 2

class Delta:
    MODE_ABSOLUTE= 0
    MODE_DIFFERENTIAL= 1
    MODE_BINARY= 2

class Pulse:
    OUTPUT_WIDTH= 0
    OUTPUT_AREA= 1
    OUTPUT_PEAK= 2
    OUTPUT_ON_DETECTION= 3



from ctypes import *
import uuid

class Module:
    SWITCH = 1
    LED = 2
    ACCELEROMETER = 3
    TEMPERATURE = 4
    GPIO = 5
    NEO_PIXEL = 6
    IBEACON = 7
    HAPTIC = 8
    DATA_PROCESSOR = 9
    EVENT = 0xa
    LOGGING = 0xb
    TIMER = 0xc
    I2C = 0xd
    MACRO = 0xf
    GSR = 0x10
    SETTINGS = 0x11
    BAROMETER = 0x12
    GYRO = 0x13
    AMBIENT_LIGHT = 0x14
    MAGNETOMETER = 0x15
    HUMIDITY = 0x16
    COLOR_DETECTOR = 0x17
    PROXIMITY = 0x18
    SENSOR_FUSION = 0x19
    DEBUG = 0xfe

# Constants identifying the message data types
class DataTypeId:
    UINT32= 0
    FLOAT= 1
    CARTESIAN_FLOAT= 2
    INT32= 3
    BYTE_ARRAY= 4
    BATTERY_STATE= 5
    TCS34725_ADC= 6
    EULER_ANGLES= 7
    QUATERNION= 8
    CORRECTED_CARTESIAN_FLOAT= 9

# Python wrapper for the MblMwMessage struct
class Data(Structure):
    _fields_= [
        ("epoch", c_longlong),
        ("value", c_void_p),
        ("type_id", c_int),
        ("length", c_ubyte)
    ]

class GattCharacteristic(Structure):
    _fields_= [
        ("service_uuid_high", c_ulonglong),
        ("service_uuid_low", c_ulonglong),
        ("uuid_high", c_ulonglong),
        ("uuid_low", c_ulonglong)
    ]

Fn_ByteArray= CFUNCTYPE(None, POINTER(c_ubyte), c_ubyte)
Fn_VoidPtr= CFUNCTYPE(None, c_void_p)
Fn_VoidPtr_Int= CFUNCTYPE(None, c_void_p, c_int)
Fn_DataPtr= CFUNCTYPE(None, POINTER(Data))
Fn_VoidPtr_GattCharPtr_ByteArray= CFUNCTYPE(None, c_void_p, POINTER(GattCharacteristic), POINTER(c_ubyte), c_ubyte)
Fn_VoidPtr_GattCharPtr= CFUNCTYPE(None, c_void_p, POINTER(GattCharacteristic))
Fn_Uint_Uint= CFUNCTYPE(None, c_uint, c_uint)
Fn_Ubyte_LongLong_ByteArray= CFUNCTYPE(None, c_ubyte, c_longlong, POINTER(c_ubyte), c_ubyte)

# UUIDs for the MetaWear gatt services and characteristics
class Gatt:
    SERVICE_UUID= uuid.UUID('{326A9000-85CB-9195-D9DD-464CFBBAE75A}')
    CHAR_NOTIFICATION_UUID= uuid.UUID('{326A9006-85CB-9195-D9DD-464CFBBAE75A}')

class BtleConnection(Structure):
    _fields_= [
        ("write_gatt_char", Fn_VoidPtr_GattCharPtr_ByteArray),
        ("read_gatt_char", Fn_VoidPtr_GattCharPtr)
    ]

# Constants indicating status codes
class Status:
    OK= 0
    WARNING_UNEXPECTED_SENSOR_DATA= 1
    WARNING_INVALID_PROCESSOR_TYPE= 2
    ERROR_UNSUPPORTED_PROCESSOR = 4
    WARNING_INVALID_RESPONSE = 8
    ERROR_TIMEOUT = 16

class LogDownloadHandler(Structure):
    _fields_= [
        ("received_progress_update", Fn_Uint_Uint),
        ("received_unknown_entry", Fn_Ubyte_LongLong_ByteArray),
        ("received_unhandled_entry", Fn_DataPtr)
    ]

class Settings:
    BATTERY_VOLTAGE_INDEX = 0
    BATTERY_CHARGE_INDEX = 1

# taken from https://www.python.org/dev/peps/pep-0485/#proposed-implementation
def is_close(fst, snd):
    return abs(fst - snd) <= max( 0.001 * max(abs(fst), abs(snd)), 0.001 )

# Python wrapper for the MblMwDataCartesianFloat struct
class CartesianFloat(Structure):
    _fields_= [
        ("x", c_float),
        ("y", c_float),
        ("z", c_float)
    ]

    def __eq__(self, other):
        return is_close(self.x, other.x) and is_close(self.y, other.y) and is_close(self.z, other.z)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        return "(%.3f, %.3f, %.3f)" % (self.x, self.y, self.z)

    def __deepcopy__(self, memo):
        return CartesianFloat(x= self.x, y= self.y, z= self.z)

class BatteryState(Structure):
    _fields_= [
        ("voltage", c_ushort),
        ("charge", c_ubyte)
    ]

    def __eq__(self, other):
        return (self.voltage == other.voltage) and (self.charge == other.charge)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        return "{voltage: %d, charge: %d}" % (self.voltage, self.charge)

    def __deepcopy__(self, memo):
        return BatteryState(voltage= self.voltage, charge= self.charge)

class Tcs34725ColorAdc(Structure):
    _fields_= [
        ("clear", c_ushort),
        ("red", c_ushort),
        ("green", c_ushort),
        ("blue", c_ushort)
    ]

    def __eq__(self, other):
        return (self.clear == other.clear) and (self.red == other.red) and (self.green == other.green) and (self.blue == other.blue)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        return "{clear: %d, red: %d, green: %d, blue: %d}" % (self.clear, self.red, self.green, self.blue)

    def __deepcopy__(self, memo):
        return Tcs34725ColorAdc(clear= self.clear, red= self.red, green= self.green, blue= self.blue)

class EulerAngle(Structure):
    _fields_= [
        ("heading", c_float),
        ("pitch", c_float),
        ("roll", c_float),
        ("yaw", c_float)
    ]

    def __eq__(self, other):
        return is_close(self.heading, other.heading) and is_close(self.pitch, other.pitch) and is_close(self.roll, other.roll) and is_close(self.yaw, other.yaw)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        return "(%.3f, %.3f, %.3f, %.3f)" % (self.heading, self.pitch, self.roll, self.yaw)

    def __deepcopy__(self, memo):
        return EulerAngle(heading = self.heading, pitch = self.pitch, roll = self.roll, yaw = self.yaw)

class Quaternion(Structure):
    _fields_= [
        ("w", c_float),
        ("x", c_float),
        ("y", c_float),
        ("z", c_float)
    ]

    def __eq__(self, other):
        return is_close(self.w, other.w) and is_close(self.x, other.x) and is_close(self.y, other.y) and is_close(self.z, other.z)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        return "(%.3f, %.3f, %.3f, %.3f)" % (self.w, self.x, self.y, self.z)

    def __deepcopy__(self, memo):
        return Quaternion(w = self.w, x = self.x, y = self.y, z = self.z)

class CorrectedCartesianFloat(Structure):
    _fields_= [
        ("x", c_float),
        ("y", c_float),
        ("z", c_float),
        ("accuracy", c_ubyte)
    ]

    def __eq__(self, other):
        return is_close(self.x, other.x) and is_close(self.y, other.y) and is_close(self.z, other.z) and self.accuracy == other.accuracy

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        return "(%.3f, %.3f, %.3f, %d)" % (self.x, self.y, self.z, self.accuracy)

    def __deepcopy__(self, memo):
        return CorrectedCartesianFloat(x = self.x, y = self.y, z = self.z, accuracy = self.accuracy)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
version.py
-----------

:copyright: 2016-11-28 by hbldh <henrik.blidh@nedomkull.com>

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

__version__ = '0.7.2'
version = __version__  # backwards compatibility name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: mgeorgi <marcus.georgi@kinemic.de>

Created: 2016-02-01

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time
import logging
from functools import wraps
from ctypes import cast, POINTER

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear import sensor, processor
from pymetawear.mbientlab.metawear.core import DataTypeId, CartesianFloat, \
    Quaternion, CorrectedCartesianFloat, EulerAngle
from pymetawear.mbientlab.metawear.core import Fn_DataPtr, Fn_VoidPtr

from pymetawear.modules.base import PyMetaWearModule, Modules

log = logging.getLogger(__name__)
current_processor = None
waiting_for_processor = False
max_processor_wait_time = 5


def require_fusion_module(f):
    def wrapper(*args, **kwargs):
        if getattr(args[0], 'available', False) is False:
            raise PyMetaWearException("There is no Sensor Fusion "
                                      "module on your MetaWear board!")
        return f(*args, **kwargs)

    return wrapper


class SensorFusionModule(PyMetaWearModule):
    """MetaWear accelerometer module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of the sensorfusion
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id, debug=False):
        super(SensorFusionModule, self).__init__(board, debug)
        self.module_id = module_id

        if self.module_id == Modules.MBL_MW_MODULE_NA:
            # No sensor fusion present!
            self.available = False
        else:
            self.available = True

        self.board = board

        self.current_active_signal = None

        self._streams_to_enable = {
            sensor.SensorFusion.DATA_CORRECTED_ACC: False,
            sensor.SensorFusion.DATA_CORRECTED_GYRO: False,
            sensor.SensorFusion.DATA_CORRECTED_MAG: False,
            sensor.SensorFusion.DATA_QUATERION: False,
            sensor.SensorFusion.DATA_EULER_ANGLE: False,
            sensor.SensorFusion.DATA_GRAVITY_VECTOR: False,
            sensor.SensorFusion.DATA_LINEAR_ACC: False,
        }

        self._data_source_signals = {
            sensor.SensorFusion.DATA_CORRECTED_ACC: None,
            sensor.SensorFusion.DATA_CORRECTED_GYRO: None,
            sensor.SensorFusion.DATA_CORRECTED_MAG: None,
            sensor.SensorFusion.DATA_QUATERION: None,
            sensor.SensorFusion.DATA_EULER_ANGLE: None,
            sensor.SensorFusion.DATA_GRAVITY_VECTOR: None,
            sensor.SensorFusion.DATA_LINEAR_ACC: None,
        }

        self._callbacks = {}
        self._waiting_for_processor = False

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0}".format(self.module_name)

    def __repr__(self):
        return str(self)

    @require_fusion_module
    def set_sample_delay(self, data_source, delay=None, differential=False):
        """
        Change the delay between samples using the onboard time processor
        module to change the effective sampling rate of a specific data source.

        :param data_source: A data source from sensor.SensorFusion
        :param delay: The delay in ms between samples,
            or None to reset to default
        :param differential: Set Time Preprocessor mode to differential,
            instead of the default, absolute
        """

        global current_processor
        global waiting_for_processor

        if self._data_source_signals[data_source] is None:
            log.debug("Getting data signal for data source {0}".format(
                data_source
            ))
            self._data_source_signals[data_source] = \
                libmetawear.mbl_mw_sensor_fusion_get_data_signal(
                    self.board, data_source
                )

        if delay is not None:
            mode = processor.Time.MODE_DIFFERENTIAL if differential else \
                processor.Time.MODE_ABSOLUTE
            waiting_for_processor = True
            log.debug("Creating time dataprocessor for signal {0}".format(
                self._data_source_signals[data_source]
            ))
            libmetawear.mbl_mw_dataprocessor_time_create(
                self._data_source_signals[data_source],
                mode,
                delay,
                Fn_VoidPtr(processor_set))

            wait_time = 0
            while waiting_for_processor and wait_time < max_processor_wait_time:
                sleeptime = 0.1
                time.sleep(sleeptime)
                wait_time += sleeptime
            if current_processor is not None:
                self._data_source_signals[data_source] = current_processor
                current_processor = None
            else:
                raise PyMetaWearException("Can't set data processor!")

        else:
            data_signal = libmetawear.mbl_mw_sensor_fusion_get_data_signal(
                    self.board, data_source)
            if self._data_source_signals[data_source] != data_signal:
                libmetawear.mbl_mw_dataprocessor_remove(
                    self._data_source_signals[data_source]
                )
                self._data_source_signals[data_source] = data_signal

    def _delay_set(self, processor):
        self._current_processor = processor
        self._waiting_for_processor = False

    @require_fusion_module
    def set_mode(self, mode):
        libmetawear.mbl_mw_sensor_fusion_set_mode(self.board,
                                                  mode)
        libmetawear.mbl_mw_sensor_fusion_write_config(self.board)

    @require_fusion_module
    def set_acc_range(self, acc_range):
        libmetawear.mbl_mw_sensor_fusion_set_acc_range(self.board,
                                                       acc_range)
        libmetawear.mbl_mw_sensor_fusion_write_config(self.board)

    @require_fusion_module
    def set_gyro_range(self, gyro_range):
        libmetawear.mbl_mw_sensor_fusion_set_gyro_range(self.board,
                                                        gyro_range)
        libmetawear.mbl_mw_sensor_fusion_write_config(self.board)

    @require_fusion_module
    def get_data_signal(self, data_source):
        if self._data_source_signals[data_source] is None:
            self._data_source_signals[data_source] = \
                libmetawear.mbl_mw_sensor_fusion_get_data_signal(
                    self.board, data_source
                )
        return self._data_source_signals[data_source]

    @property
    def data_signal(self):
        return self.current_active_signal

    @property
    def module_name(self):
        return "Sensor Fusion"

    def get_current_settings(self):
        raise NotImplementedError()

    def get_possible_settings(self):
        raise NotImplementedError()

    def set_settings(self):
        raise NotImplementedError()

    @require_fusion_module
    def notifications(self,
                      corrected_acc_callback=None,
                      corrected_gyro_callback=None,
                      corrected_mag_callback=None,
                      quaternion_callback=None,
                      euler_angle_callback=None,
                      gravity_callback=None,
                      linear_acc_callback=None):
        """Subscribe or unsubscribe to sensor fusion notifications.

        Convenience method for handling sensor fusion usage.

        Example:

        .. code-block:: python

            def handle_notification(data):
                # Handle a (epoch_time, (x,y,z,accuracy)) corrected acc tuple.
                epoch = data[0]
                xyzaccu = data[1]
                print("[{0}] X: {1}, Y: {2}, Z: {3}".format(
                    epoch, *xyzaccu[:-1]))

            mwclient.sensorfusion.notifications(
                corrected_acc_callback=handle_notification)

        :param callable corrected_acc_callback: Acceleration notification
            callback function.
            If `None`, unsubscription to acceleration notifications is
            registered.
        :param callable corrected_gyro_callback: Gyroscope notification
            callback function.
            If `None`, unsubscription to gyroscope notifications is registered.
        :param callable corrected_mag_callback: Magnetometer notification
            callback function.
            If `None`, unsubscription to magnetometer notifications is
            registered.
        :param callable quaternion_callback: Quaternion notification callback
            function.
            If `None`, unsubscription to quaternion notifications is registered.
        :param callable euler_angle_callback: Euler angle notification callback
            function.
            If `None`, unsubscription to euler angle notifications is
            registered.
        :param callable gravity_callback: Gravity vector notification callback
            function.
            If `None`, unsubscription to gravity notifications is registered.
        :param callable linear_acc_callback: Linear acceleration notification
            callback function.
            If `None`, unsubscription to linear acceleration notifications is
            registered.

        """
        callback_data_source_map = {
            sensor.SensorFusion.DATA_CORRECTED_ACC: corrected_acc_callback,
            sensor.SensorFusion.DATA_CORRECTED_GYRO: corrected_gyro_callback,
            sensor.SensorFusion.DATA_CORRECTED_MAG: corrected_mag_callback,
            sensor.SensorFusion.DATA_QUATERION: quaternion_callback,
            sensor.SensorFusion.DATA_EULER_ANGLE: euler_angle_callback,
            sensor.SensorFusion.DATA_GRAVITY_VECTOR: gravity_callback,
            sensor.SensorFusion.DATA_LINEAR_ACC: linear_acc_callback
        }

        for data_source in callback_data_source_map:
            if callback_data_source_map[data_source] is not None:
                self._streams_to_enable[data_source] = True
            else:
                self._streams_to_enable[data_source] = False

        enable = False in [x is None for x in callback_data_source_map.values()]
        log.debug("Enable: %s" % enable)

        if not enable:
            self.stop()
            self.toggle_sampling(False)

        for data_source in callback_data_source_map:
            self.current_active_signal = self.get_data_signal(data_source)
            callback = callback_data_source_map[data_source]
            if callback is not None:
                self.check_and_change_callback(
                    self.get_data_signal(data_source),
                    sensor_data(callback)
                )
            else:
                self.check_and_change_callback(
                    self.get_data_signal(data_source),
                    None
                )

        if enable:
            self.toggle_sampling(True)
            self.start()

    def check_and_change_callback(self, data_signal, callback):
        if callback is not None:
            if self._debug:
                log.debug("Subscribing to {0} changes. (Sig#: {1})".format(
                    self.module_name, data_signal))
            if data_signal in self._callbacks:
                raise PyMetaWearException(
                    "Subscription to {0} signal already in place!")
            self._callbacks[data_signal] = (callback, Fn_DataPtr(callback))
            libmetawear.mbl_mw_datasignal_subscribe(
                data_signal, self._callbacks[data_signal][1])
        else:
            if data_signal not in self._callbacks:
                return
            if self._debug:
                log.debug("Unsubscribing to {0} changes. (Sig#: {1})".format(
                    self.module_name, data_signal))
            libmetawear.mbl_mw_datasignal_unsubscribe(data_signal)
            del self._callbacks[data_signal]

    @require_fusion_module
    def start(self):
        """Switches the sensorfusion to active mode."""
        libmetawear.mbl_mw_sensor_fusion_start(self.board)

    @require_fusion_module
    def stop(self):
        """Switches the sensorfusion to standby mode."""
        libmetawear.mbl_mw_sensor_fusion_stop(self.board)

    @require_fusion_module
    def toggle_sampling(self, enabled=True):
        """Enables or disables sensor fusion sampling.

        :param bool enabled: Desired state of the sensor fusion module.

        """
        if enabled:
            for data_source in self._streams_to_enable:
                if self._streams_to_enable[data_source]:
                    libmetawear.mbl_mw_sensor_fusion_enable_data(
                        self.board, data_source)
        else:
            libmetawear.mbl_mw_sensor_fusion_clear_enabled_mask(
                self.board)


def processor_set(processor):
    """
    Set global variables as the libmetawear callback can't handle the self
    parameter of instance methods.

    :param processor: The processor thas was created
    """
    global current_processor
    global waiting_for_processor
    current_processor = processor
    waiting_for_processor = False


def sensor_data(func):
    @wraps(func)
    def wrapper(data):
        if data.contents.type_id == DataTypeId.CARTESIAN_FLOAT:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(CartesianFloat))
            func((epoch, (data_ptr.contents.x,
                          data_ptr.contents.y,
                          data_ptr.contents.z)))
        elif data.contents.type_id == DataTypeId.QUATERNION:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(Quaternion))
            func((epoch, (data_ptr.contents.w,
                          data_ptr.contents.x,
                          data_ptr.contents.y,
                          data_ptr.contents.z)))
        elif data.contents.type_id == DataTypeId.CORRECTED_CARTESIAN_FLOAT:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value,
                            POINTER(CorrectedCartesianFloat))
            func((epoch, (data_ptr.contents.x,
                          data_ptr.contents.y,
                          data_ptr.contents.z,
                          data_ptr.contents.accuracy)))
        elif data.contents.type_id == DataTypeId.EULER_ANGLES:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(EulerAngle))
            func((epoch, (data_ptr.contents.heading,
                          data_ptr.contents.pitch,
                          data_ptr.contents.roll,
                          data_ptr.contents.yaw)))
        else:
            raise PyMetaWearException('Incorrect data type id: {0}'.format(
                data.contents.type_id))

    return wrapper



from ctypes import c_ushort, c_ubyte, Structure

# Constants and types for configuring the Led
class Led:
    # Python wrapper for the MblMwLedPattern struct
    class Pattern(Structure):
        _fields_= [
            ("high_intensity", c_ubyte),
            ("low_intensity", c_ubyte),
            ("rise_time_ms", c_ushort),
            ("high_time_ms", c_ushort),
            ("fall_time_ms", c_ushort),
            ("pulse_duration_ms", c_ushort),
            ("delay_time_ms", c_ushort),
            ("repeat_count", c_ubyte)
        ]

    REPEAT_INDEFINITELY= 0xff

    COLOR_GREEN= 0
    COLOR_RED= 1
    COLOR_BLUE= 2

    PRESET_BLINK= 0
    PRESET_PULSE= 1
    PRESET_SOLID= 2

# Constants for configuring NeoPixel strands
class NeoPixel:
    COLOR_ORDERING_WS2811_RGB= 0
    COLOR_ORDERING_WS2811_RBG= 1
    COLOR_ORDERING_WS2811_GRB= 2
    COLOR_ORDERING_WS2811_GBR= 3

    ROT_DIRECTION_TOWARDS= 0
    ROT_DIRECTION_AWAY= 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`discover`
---------------

:copyright: 2016-08-09 by hbldh <henrik.blidh@swedwise.com>

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

from pymetawear.discover import discover_devices

try:
    input_fcn = raw_input
except NameError:
    input_fcn = input


def scan_and_select_le_device(timeout=3):
    print("Discovering nearby Bluetooth Low Energy devices...")
    ble_devices = discover_devices(timeout=timeout)
    if len(ble_devices) > 1:
        for i, d in enumerate(ble_devices):
            print("[{0}] - {1}: {2}".format(i+1, *d))
        s = input("Which device do you want to connect to? ")
        if int(s) <= (i + 1):
            address = ble_devices[int(s) - 1][0]
        else:
            raise ValueError("Incorrect selection. Aborting...")
    elif len(ble_devices) == 1:
        address = ble_devices[0][0]
    else:
        raise ValueError("Did not detect any BLE devices.")
    return address



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created: 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import logging
from functools import wraps
from ctypes import c_uint, cast, POINTER

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear.core import DataTypeId
from pymetawear.modules.base import PyMetaWearModule

log = logging.getLogger(__name__)


class SwitchModule(PyMetaWearModule):
    """MetaWear Switch module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, debug=False):
        super(SwitchModule, self).__init__(board, debug)

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0}".format(
            self.module_name)

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Switch"

    @property
    def data_signal(self):
        return libmetawear.mbl_mw_switch_get_state_data_signal(self.board)

    def notifications(self, callback=None):
        """Subscribe or unsubscribe to switch notifications.

        Convenience method for handling switch usage.

        Example:

        .. code-block:: python

            def switch_callback(data):
                epoch = data[0]
                status = data[1]
                if status == 1:
                    print("[{0}] Switch pressed!".format(epoch))
                elif status == 0:
                    print("[{0}] Switch released!".format(epoch))

            mwclient.switch.notifications(switch_callback)

        :param callable callback: Switch notification callback function.
            If `None`, unsubscription to switch notifications is registered.

        """
        super(SwitchModule, self).notifications(
            switch_data(callback) if callback is not None else None)


def switch_data(func):
    @wraps(func)
    def wrapper(data):
        if data.contents.type_id == DataTypeId.UINT32:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(c_uint))
            func((epoch, data_ptr.contents.value))
        else:
            raise PyMetaWearException('Incorrect data type id: {0}'.format(
                data.contents.type_id))
    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created: 2016-08-16

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import re
import logging
from functools import wraps
from ctypes import c_float, cast, POINTER

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear import sensor
from pymetawear.mbientlab.metawear.core import DataTypeId
from pymetawear.modules.base import PyMetaWearModule

log = logging.getLogger(__name__)


class BarometerModule(PyMetaWearModule):
    """MetaWear barometer module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of this barometer
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id, debug=False):
        super(BarometerModule, self).__init__(board, debug)
        self.module_id = module_id
        self._altitude_data = True

        barometer_sensors = {
            0: sensor.BarometerBmp280,
            1: sensor.BarometerBme280,
        }
        self.barometer_class = barometer_sensors.get(self.module_id, None)

        if self.barometer_class is not None:
            self.oversampling = {"".join(re.search(
                '^OVERSAMPLING_([A-Z\_]*)', k).groups()).lower():
                getattr(sensor.BarometerBosch, k, None) for k in filter(
                lambda x: x.startswith('OVERSAMPLING'),
                vars(sensor.BarometerBosch))}
            for k in self.oversampling:
                if isinstance(self.oversampling[k], tuple):
                    self.oversampling[k] = self.oversampling[k][0]

            self.iir_filter = {"".join(re.search(
                '^IIR_FILTER_([0-9AVG\_OFF]+)', k).groups()).lower():
                getattr(sensor.BarometerBosch, k, None) for k in filter(
                lambda x: x.startswith('IIR_FILTER'),
                vars(sensor.BarometerBosch))}

            self.standby_time = {float(".".join(re.search(
                '^STANDBY_TIME_([0-9]+)\_*([0-9]*)MS', k).groups())):
                getattr(self.barometer_class, k, None) for k in filter(
                lambda x: x.startswith('STANDBY_TIME'),
                vars(self.barometer_class))}
        else:
            self.is_present = False

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0} {1}".format(
            self.module_name, self.sensor_name)

    def __repr__(self):
        return str(self)

    def set_altitude_data(self, status=True):
        """Change between altitude and pressure output.

        :param bool status:

        """
        self._altitude_data = bool(status)

    @property
    def module_name(self):
        return "Barometer"

    @property
    def sensor_name(self):
        return self.barometer_class.__name__.replace('Barometer', '')

    @property
    def data_signal(self):
        if self._altitude_data:
            return libmetawear.mbl_mw_baro_bosch_get_altitude_data_signal(
                self.board)
        else:
            return libmetawear.mbl_mw_baro_bosch_get_pressure_data_signal(
                self.board)

    def _get_oversampling(self, value):
        if value.lower() in self.oversampling:
            return self.oversampling.get(value.lower())
        else:
            raise ValueError(
                "Requested oversampling ({0}) was not part of possible values: {1}".format(
                    value.lower(), self.oversampling.keys()))

    def _get_iir_filter(self, value):
        if value.lower() in self.iir_filter:
            return self.iir_filter.get(value.lower())
        else:
            raise ValueError(
                "Requested IIR filter ({0}) was not part of possible values: {1}".format(
                    value.lower(), self.iir_filter.keys()))

    def _get_standby_time(self, value):
        sorted_ord_keys = sorted(self.standby_time.keys(),
                                 key=lambda x: (float(x)))
        diffs = [abs(value - float(k)) for k in sorted_ord_keys]
        min_diffs = min(diffs)
        if min_diffs > 0.5:
            raise ValueError(
                "Requested standby time ({0}) was not part of possible values: {1}".format(
                    value, [float(x) for x in sorted_ord_keys]))
        return float(value)

    def get_current_settings(self):
        raise NotImplementedError()

    def get_possible_settings(self):
        return {
            'oversampling': [x.lower() for x in sorted(
                self.oversampling.keys(), key=lambda x: (x.lower()))],
            'iir_filter': [x.lower() for x in sorted(
                self.iir_filter.keys(), key=lambda x: (x.lower()))],
            'standby_time': [float(x) for x in sorted(
                self.standby_time.keys(), key=lambda x: (float(x)))],
        }

    def set_settings(self, oversampling=None, iir_filter=None,
                     standby_time=None):
        """Set barometer settings.

         Can be called with 1-3 setting:

         .. code-block:: python

            mwclient.set_settings(oversampling='low_power',
                                  iir_filter='avg_2',
                                  standby_time=125.0)

        will give the same result as

        .. code-block:: python

            mwclient.set_settings(oversampling='low_power')
            mwclient.set_settings(iir_filter='avg_2')
            mwclient.set_settings(standby_time=125.0)

        albeit that the latter example makes two writes to the board.

        Call :meth:`~get_possible_settings` to see which values
        that can be set for this sensor.

        :param str oversampling:
        :param str iir_filter:
        :param float standby_time:

        """
        if oversampling is not None:
            oversampling = self._get_oversampling(oversampling)
            if self._debug:
                log.debug("Setting Barometer Oversampling to {0}".format(
                    oversampling))
            libmetawear.mbl_mw_baro_bosch_set_oversampling(
                self.board, oversampling)
        if iir_filter is not None:
            iir_filter = self._get_iir_filter(iir_filter)
            if self._debug:
                log.debug(
                    "Setting Barometer IIR filter to {0}".format(iir_filter))
            libmetawear.mbl_mw_baro_bosch_set_iir_filter(
                self.board, iir_filter)
        if standby_time is not None:
            standby_time = self._get_standby_time(standby_time)
            if self._debug:
                log.debug("Setting Barometer Standby Time to {0}".format(
                    standby_time))
            libmetawear.mbl_mw_baro_bosch_set_standby_time(
                self.board, standby_time)

        if (oversampling is not None) or (iir_filter is not None) or \
                (standby_time is not None):
            libmetawear.mbl_mw_baro_bosch_write_config(self.board)

    def notifications(self, callback=None):
        """Subscribe to or unsubscribe from barometer notifications.

        Convenience method for handling barometer usage.

        Example:

        .. code-block:: python

            def handle_barometer_notification(data)
                # Handle a (epoch_time, value) barometer tuple.
                epoch = data[0]
                value = data[1]
                print("[{0}] Altitude: {1}".format(epoch, value))

            mwclient.barometer.notifications(handle_barometer_notification)

        :param callable callback: Barometer notification callback function.
            If `None`, unsubscription to barometer notifications is registered.

        """

        if callback is None:
            self.stop()
            super(BarometerModule, self).notifications(None)
        else:
            super(BarometerModule, self).notifications(
                sensor_data(callback))
            self.start()

    def start(self):
        """Switches the barometer to active mode."""
        libmetawear.mbl_mw_baro_bosch_start(self.board)

    def stop(self):
        """Switches the barometer to standby mode."""
        libmetawear.mbl_mw_baro_bosch_stop(self.board)


def sensor_data(func):
    @wraps(func)
    def wrapper(data):
        if data.contents.type_id == DataTypeId.FLOAT:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(c_float))
            func((epoch, data_ptr.contents.value))
        else:
            raise PyMetaWearException('Incorrect data type id: {0}'.format(
                data.contents.type_id))

    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`led`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.client import MetaWearClient
from pymetawear.discover import select_device

address = select_device()
c = MetaWearClient(str(address), debug=True)
print("New client created: {0}".format(c))

print("Blinking 10 times with green LED...")
from ctypes import byref
from pymetawear import libmetawear
from pymetawear.mbientlab.metawear.peripheral import Led

pattern = Led.Pattern(repeat_count=10)
libmetawear.mbl_mw_led_load_preset_pattern(byref(pattern), Led.PRESET_BLINK)
libmetawear.mbl_mw_led_write_pattern(c.board, byref(pattern), Led.COLOR_GREEN)
libmetawear.mbl_mw_led_play(c.board)

time.sleep(5.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
The setup script for the PyMetaWear package.

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created on 2016-04-08

'''

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import os
import sys
import platform
import subprocess
import shutil
import re
import glob
from codecs import open

from setuptools import setup, find_packages
from setuptools.command import build_py, develop

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()


class PyMetaWearDeveloper(develop.develop):
    def run(self):
        build_solution()
        develop.develop.run(self)


class PyMetaWearBuilder(build_py.build_py):
    def run(self):
        build_solution()
        build_py.build_py.run(self)


def build_solution():
    # Establish source paths.
    basedir = os.path.abspath(os.path.dirname(__file__))
    pkg_dir = os.path.join(basedir, 'pymetawear')
    path_to_metawear_python_wrappers = os.path.join(
        pkg_dir, 'Metawear-CppAPI', 'wrapper', 'python')

    if os.path.exists(os.path.join(basedir, '.git')):
        # The package was cloned from Github and the submodule can
        # therefore be brought in by Git methods.

        # Git submodule init
        p = subprocess.Popen(['git', 'submodule', 'init'],
                             cwd=basedir, stdout=sys.stdout, stderr=sys.stderr)
        p.communicate()

        # Git submodule update
        p = subprocess.Popen(['git', 'submodule', 'update'],
                             cwd=basedir, stdout=sys.stdout, stderr=sys.stderr)
        p.communicate()
    else:
        # The package was downloaded as zip or tar.gz from PyPI. It should
        # have the MetaWear-CppAPI folder bundled and the building can be done immediately.
        pass

    if platform.uname()[0] == 'Linux':
        arch = os.uname()[-1]
        if arch in ('x86_64', 'amd64'):
            dist_dir = 'x64'
        elif 'arm' in arch:
            dist_dir = 'arm'
        else:
            dist_dir = 'x86'

        # Run make file for MetaWear-CppAPI
        p = subprocess.Popen(
            ['make', 'clean'],
            cwd=os.path.join(pkg_dir, 'Metawear-CppAPI'),
            stdout=sys.stdout, stderr=sys.stderr)
        p.communicate()
        p = subprocess.Popen(
            ['make', 'build'],
            cwd=os.path.join(pkg_dir, 'Metawear-CppAPI'),
            stdout=sys.stdout, stderr=sys.stderr)
        p.communicate()

        path_to_dist_dir = os.path.join(
            pkg_dir, 'Metawear-CppAPI', 'dist', 'release', 'lib', dist_dir)

        for f in [s for s in os.listdir(pkg_dir) if s.startswith('libmetawear')]:
            os.remove(os.path.join(pkg_dir, f))

        symlinks_to_create = []
        # Copy the built shared library to pymetawear folder.
        for dist_file in glob.glob(path_to_dist_dir + "/libmetawear.*"):
            if os.path.islink(dist_file):
                symlinks_to_create.append(
                    (os.path.basename(os.readlink(dist_file)),
                     os.path.basename(dist_file)))
            else:
                destination_file = os.path.join(
                    pkg_dir, os.path.basename(dist_file))
                shutil.copy(dist_file, destination_file)

        # Create symlinks for the libmetawear shared library.
        for symlink_src, symlink_dest in symlinks_to_create:
            destination_symlink = os.path.join(pkg_dir, symlink_dest)
            os.symlink(symlink_src, destination_symlink)

    elif platform.uname()[0] == 'Windows':
        arch = platform.architecture()[0]
        if arch == '32bit':
            dist_dir = 'Win32'
            msbuild_file = 'MetaWear.Win32.vcxproj'
            build_options = '/p:Configuration=Release;Platform=Win32'
        elif 'arm' in arch:
            dist_dir = 'ARM'
            msbuild_file = 'MetaWear.WinRT.vcxproj'
            build_options = '/p:Configuration=Release;Platform=ARM'
        else:
            dist_dir = 'x64'
            msbuild_file = 'MetaWear.Win32.vcxproj'
            build_options = '/p:Configuration=Release;Platform=x64'

        # Run msbuild file for MetaWear-CppAPI
        vsvars_file = glob.glob('c:\\Progr*/**/**/Too*/vsvars32.bat')[0]
        p = subprocess.Popen('"{0}" & MSBuild.exe {1} {2}'.format(
            vsvars_file, msbuild_file, build_options),
            cwd=os.path.join(pkg_dir, 'Metawear-CppAPI'),
            stdout=sys.stdout, stderr=sys.stderr)
        p.communicate()

        for f in [s for s in os.listdir(pkg_dir) if (s.startswith('MetaWear') and s.endswith('.dll'))]:
            os.remove(os.path.join(pkg_dir, f))

        path_to_dist_dir = os.path.join(
            pkg_dir, 'Metawear-CppAPI', 'dist', 'Release', 'lib', dist_dir)

        # Copy the built shared library to pymetawear folder.
        for dist_file in glob.glob(path_to_dist_dir + "/MetaWear.*.dll"):
            destination_file = os.path.join(
                pkg_dir, os.path.basename(dist_file))
            shutil.copy(dist_file, destination_file)
    else:
        raise NotImplementedError("Building on this platform is not implemented.")

    # Copy the Mbientlab Python wrappers to pymetawear folder.
    # First create folders if needed.
    try:
        os.makedirs(os.path.join(pkg_dir, 'mbientlab', 'metawear'))
    except:
        pass

    init_files_to_create = [
        os.path.join(pkg_dir, 'mbientlab', '__init__.py'),
        os.path.join(pkg_dir, 'mbientlab', 'metawear', '__init__.py')
    ]
    for init_file in init_files_to_create:
        with open(init_file, 'w') as f:
            f.write("#!/usr/bin/env python\n# -*- coding: utf-8 -*-")

    # Copy all Python files from the MetWear C++ API Python wrapper
    for pth, _, pyfiles in os.walk(
            os.path.join(path_to_metawear_python_wrappers,
                         'mbientlab', 'metawear')):
        for py_file in filter(lambda x: os.path.splitext(x)[1] == '.py', pyfiles):
            try:
                shutil.copy(
                    os.path.join(pth, py_file),
                    os.path.join(pkg_dir, 'mbientlab', 'metawear', py_file))
            except:
                pass

with open('pymetawear/version.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)


def read(f):
    return open(f, encoding='utf-8').read()


setup(
    name='pymetawear',
    version=version,
    author='Henrik Blidh',
    author_email='henrik.blidh@nedomkull.com',
    url='https://github.com/hbldh/pymetawear',
    description="Python client for connecting to MbientLab's MetaWear boards",
    long_description=read('README.rst') + '\n\n' + read('HISTORY.rst'),
    license='MIT',
    platforms=['Linux'],
    keywords=['Bluetooth', 'IMU', 'MetaWear', 'MbientLab'],
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Operating System :: POSIX :: Linux',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
    ],
    cmdclass={
        'build_py': PyMetaWearBuilder,
        'develop': PyMetaWearDeveloper
    },
    packages=find_packages(exclude=['tests', 'docs', 'examples', 'examples.*']),
    include_package_data=True,
    # Adding MbientLab's Python code as package data since it is copied
    # to folder after ``find_packages`` is run.
    package_data={
        'pymetawear': [
            'libmetawear.so*',
            'mbientlab/*.py',
            'mbientlab/metawear/*.py'
        ],
    },
    install_requires=[
        'pygatt[GATTTOOL]>=3.0.0',
        'pexpect>=4.2.0'
    ],
    extras_require={
        'pybluez': 'pybluez[ble]>=0.22'
    },
    ext_modules=[],
    entry_points={
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created: 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import logging

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear.core import Fn_DataPtr

log = logging.getLogger(__name__)


class Modules(object):
    """Class for storing PyMetaWear module identifiers."""

    MBL_MW_MODULE_NA = -1
    MBL_MW_MODULE_SWITCH = 1
    MBL_MW_MODULE_LED = 2
    MBL_MW_MODULE_ACCELEROMETER = 3
    MBL_MW_MODULE_TEMPERATURE = 4
    MBL_MW_MODULE_GPIO = 5
    MBL_MW_MODULE_NEO_PIXEL = 6
    MBL_MW_MODULE_IBEACON = 7
    MBL_MW_MODULE_HAPTIC = 8
    MBL_MW_MODULE_DATA_PROCESSOR = 9
    MBL_MW_MODULE_EVENT = 0xa
    MBL_MW_MODULE_LOGGING = 0xb
    MBL_MW_MODULE_TIMER = 0xc
    MBL_MW_MODULE_I2C = 0xd
    MBL_MW_MODULE_MACRO = 0xf
    MBL_MW_MODULE_GSR = 0x10
    MBL_MW_MODULE_SETTINGS = 0x11
    MBL_MW_MODULE_BAROMETER = 0x12
    MBL_MW_MODULE_GYRO = 0x13
    MBL_MW_MODULE_AMBIENT_LIGHT = 0x14
    MBL_MW_MODULE_MAGNETOMETER = 0x15
    MBL_MW_MODULE_HUMIDITY = 0x16
    MBL_MW_MODULE_COLOR_DETECTOR = 0x17
    MBL_MW_MODULE_PROXIMITY = 0x18
    MBL_MW_MODULE_SENSOR_FUSION = 0x19
    MBL_MW_MODULE_DEBUG = 0xfe


class PyMetaWearModule(object):
    """Base class for PyMetaWear module implementations.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, debug=False):
        self.board = board
        self._debug = debug
        self.is_present = True
        self.callback = None

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "PyMetaWear module: {0}".format()

    def __repr__(self):
        return super(PyMetaWearModule, self).__repr__()

    @property
    def module_name(self):
        """Get module name.

        :return: The name of this module.
        :rtype: str

        """
        return ''

    @property
    def sensor_name(self):
        """Get sensor name, if applicable.

        :return: The name of this module.
        :rtype: str or None

        """
        return None

    @property
    def data_signal(self):
        """Returns the data signal pointer value for the switch module.

        :returns: The pointer value. (Long if on x64 architecture.)
        :rtype: :py:class:`ctypes.c_long` or :py:class:`ctypes.c_int`

        """
        raise PyMetaWearException(
            "No data signal exists for {0} module.".format(self))

    def set_settings(self, **kwargs):
        raise PyMetaWearException(
            "No settings exists for {0} module.".format(self))

    def get_current_settings(self):
        return {}

    def get_possible_settings(self):
        return {}

    def notifications(self, callback=None):
        """Toggle notifications/subscriptions to data signals
        on the MetaWear board.

        :param callable callback: The function to call when
            data signal notification arrives. If ``None``, an
            unsubscription to notifications is sent.

        """
        data_signal = self.data_signal
        if callback is not None:
            if self._debug:
                log.debug("Subscribing to {0} changes. (Sig#: {1})".format(
                    self.module_name, data_signal))
            if self.callback is not None:
                raise PyMetaWearException(
                    "Subscription to {0} signal already in place!")
            self.callback = (callback, Fn_DataPtr(callback))
            libmetawear.mbl_mw_datasignal_subscribe(
                data_signal, self.callback[1])
        else:
            if self._debug:
                log.debug("Unsubscribing to {0} changes. (Sig#: {1})".format(
                    self.module_name, data_signal))
            if self.callback is None:
                return
            libmetawear.mbl_mw_datasignal_unsubscribe(data_signal)
            self.callback = None



import copy
from common import TestMetaWearBase
from ctypes import byref
#from datetime import datetime
from logdata import *
from mbientlab.metawear.core import *
from mbientlab.metawear.sensor import AccelerometerBosch, GyroBmi160, SensorFusion
#from time import mktime
import threading

class TestLoggingModule(TestMetaWearBase):
    def test_start_overwrite(self):
        expected_cmds= [
            [0x0b, 0x0b, 0x01],
            [0x0b, 0x01, 0x01]
        ]

        self.libmetawear.mbl_mw_logging_start(self.board, 1)
        self.assertEqual(self.command_history, expected_cmds)

    def test_start_no_overwrite(self):
        expected_cmds= [
            [0x0b, 0x0b, 0x00],
            [0x0b, 0x01, 0x01]
        ]

        self.libmetawear.mbl_mw_logging_start(self.board, 0)
        self.assertEqual(self.command_history, expected_cmds)
        
    def test_stop(self):
        expected= [0x0b, 0x01, 0x00]

        self.libmetawear.mbl_mw_logging_stop(self.board)
        self.assertEqual(self.command, expected)

    def test_clear_entries(self):
        expected= [0x0b, 0x09, 0xff, 0xff, 0xff, 0xff]

        self.libmetawear.mbl_mw_logging_clear_entries(self.board)
        self.assertEqual(self.command, expected)

class TestLogDownload(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

        self.progress_update= Fn_Uint_Uint(self.progress_update_handler)
        self.unknown_entry= Fn_Ubyte_LongLong_ByteArray(self.unknown_entry_handler)
        self.download_handler= LogDownloadHandler(received_progress_update = self.progress_update, received_unknown_entry = self.unknown_entry)
        self.updates= []

    def progress_update_handler(self, entries_left, total_entries):
        self.updates.append(entries_left)
        if (entries_left == 0):
            self.assertEqual(self.updates, self.expected_updates)

    def unknown_entry_handler(self, id, epoch, data, length):
#        now= datetime.datetime.now()
#        now_ms= (mktime(now.timetuple()) + now.microsecond/1000000.0) * 1000
        data_ptr= cast(data, POINTER(c_uint))
        self.assertTrue(id == self.expected_entry[0] and data_ptr.contents.value == self.expected_entry[1])

    def test_readout_page_confirm(self):
        expected= [0x0b, 0x0e]
        page_completed= create_string_buffer(b'\x0b\x0d', 2)

        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, page_completed.raw, len(page_completed.raw))
        self.assertEqual(self.command, expected)

    def test_readout_progress(self):
        self.expected_updates= [
            0x0271, 0x0251, 0x0231, 0x0211, 0x01f1, 
            0x01d1, 0x01b1, 0x0191, 0x0171, 0x0151, 
            0x0131, 0x0111, 0x00f1, 0x00d1, 0x00b1, 
            0x0091, 0x0071, 0x0051, 0x0031, 0x0011, 0x0000
        ]

        progress_responses= [
            create_string_buffer(b'\x0b\x08\x71\x02\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x51\x02\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x31\x02\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x11\x02\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\xf1\x01\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\xd1\x01\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\xb1\x01\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x91\x01\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x71\x01\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x51\x01\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x31\x01\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x11\x01\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\xf1\x00\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\xd1\x00\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\xb1\x00\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x91\x00\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x71\x00\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x51\x00\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x31\x00\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x11\x00\x00\x00', 6),
            create_string_buffer(b'\x0b\x08\x00\x00\x00\x00', 6)
        ]

        self.libmetawear.mbl_mw_logging_download(self.board, 20, byref(self.download_handler))
        for buffer in progress_responses:
            self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, buffer.raw, len(buffer.raw))

    def test_download(self):
        expected_cmds= [
            [0x0b, 0x0d, 0x01],
            [0x0b, 0x07, 0x01],
            [0x0b, 0x08, 0x01],
            [0x0b, 0x85],
            [0x0b, 0x06, 0x9e, 0x01, 0x00, 0x00, 0x14, 0x00, 0x00, 0x00]
        ]

        self.libmetawear.mbl_mw_logging_download(self.board, 20, self.download_handler)
        self.assertEqual(self.command_history, expected_cmds)

    def test_unknown_entry(self):
        self.expected_entry= [0x1, 0x016c]
        log_response= create_string_buffer(b'\x0b\x07\xa1\xcc\x4d\x00\x00\x6c\x01\x00\x00', 11)

        self.libmetawear.mbl_mw_logging_download(self.board, 20, byref(self.download_handler))
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, log_response.raw, len(log_response.raw))

class TestAccelerometerLoggingBase(TestMetaWearBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.logged_data= []
        self.data_time_offsets= []
        self.prev_time= -1

        self.download_handler= LogDownloadHandler(received_progress_update = cast(None, Fn_Uint_Uint), \
                received_unknown_entry = cast(None, Fn_Ubyte_LongLong_ByteArray), received_unhandled_entry = cast(None, Fn_DataPtr))
        self.logger_ready= Fn_VoidPtr(self.logger_ready_handler)

    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

    def cartesian_float_data_handler(self, data):
        if (self.prev_time == -1):
            self.prev_time= data.contents.epoch
        else:
            self.data_time_offsets.append(data.contents.epoch - self.prev_time)
            self.prev_time= data.contents.epoch

        contents= copy.deepcopy(cast(data.contents.value, POINTER(CartesianFloat)).contents)
        self.logged_data.append(contents)

    def logger_ready_handler(self, logger):
        cartesian_float_data= Fn_DataPtr(self.cartesian_float_data_handler)
        self.libmetawear.mbl_mw_logger_subscribe(logger, cartesian_float_data)
        self.libmetawear.mbl_mw_logging_download(self.board, 20, byref(self.download_handler))
        for buffer in Bmi160Accelerometer.log_responses:
            self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, buffer.raw, len(buffer.raw))

class TestAccelerometerLogging(TestAccelerometerLoggingBase):
    def test_acc_data(self):
        acc_signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccelerometerBosch.FSR_8G)
        self.libmetawear.mbl_mw_datasignal_log(acc_signal, self.logger_ready)

        self.assertEqual(self.logged_data, Bmi160Accelerometer.expected_values)

    def test_epoch_calc(self):
        acc_signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccelerometerBosch.FSR_8G)
        self.libmetawear.mbl_mw_datasignal_log(acc_signal, self.logger_ready)

        self.assertEqual(self.data_time_offsets, Bmi160Accelerometer.expected_offsets)

class TestGyroYAxisLoggingBase(TestMetaWearBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.logged_data= []
        self.download_handler= LogDownloadHandler(received_progress_update = cast(None, Fn_Uint_Uint), \
                received_unknown_entry = cast(None, Fn_Ubyte_LongLong_ByteArray), received_unhandled_entry = cast(None, Fn_DataPtr))
        self.logger_ready= Fn_VoidPtr(self.logger_ready_handler)

    def setUp(self):        
        self.boardType= TestMetaWearBase.METAWEAR_RPRO_BOARD

        super().setUp()

    def float_data_handler(self, data):
        self.logged_data.append(cast(data.contents.value, POINTER(c_float)).contents.value)

    def logger_ready_handler(self, logger):
        cartesian_float_data= Fn_DataPtr(self.float_data_handler)
        self.libmetawear.mbl_mw_logger_subscribe(logger, cartesian_float_data)
        self.libmetawear.mbl_mw_logging_download(self.board, 20, byref(self.download_handler))

        byte_buffer= []
        for line in Bmi160GyroYAxis.log_responses:
            byte_buffer.append(self.to_string_buffer(line))

        for buffer in byte_buffer:
            self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, buffer.raw, len(buffer.raw))

class TestGyroYAxisLogging(TestGyroYAxisLoggingBase):
    def test_gyro_data(self):
        rot_signal= self.libmetawear.mbl_mw_gyro_bmi160_get_rotation_data_signal(self.board)
        roy_y_signal = self.libmetawear.mbl_mw_datasignal_get_component(rot_signal, GyroBmi160.ROTATION_Y_AXIS_INDEX)
        self.libmetawear.mbl_mw_gyro_bmi160_set_range(self.board, GyroBmi160.FSR_250DPS)

        self.libmetawear.mbl_mw_datasignal_log(roy_y_signal, self.logger_ready)

        # why doesn't unittest come with an assertAlmostEqual for list of floats?
        self.assertEqual(len(self.logged_data), len(Bmi160GyroYAxis.expected_values))
        for a, b in zip(self.logged_data, Bmi160GyroYAxis.expected_values):
            self.assertAlmostEqual(a, b, delta = 0.001)

class TestSensorFusionLogging(TestMetaWearBase):
    def logger_ready_handler(self, logger):
        self.logger = logger

    def setUp(self):
        self.boardType = TestMetaWearBase.METAWEAR_MOTION_R_BOARD
        self.logger_ready = Fn_VoidPtr(self.logger_ready_handler)

        super().setUp()

    def test_quaternion_setup(self):
        expected_cmds= [
            [0x0b, 0x02, 0x19, 0x07, 0xff, 0x60],
            [0x0b, 0x02, 0x19, 0x07, 0xff, 0x64],
            [0x0b, 0x02, 0x19, 0x07, 0xff, 0x68],
            [0x0b, 0x02, 0x19, 0x07, 0xff, 0x6c]
        ]

        signal = self.libmetawear.mbl_mw_sensor_fusion_get_data_signal(self.board, SensorFusion.DATA_QUATERION)
        self.libmetawear.mbl_mw_datasignal_log(signal, self.logger_ready)

        self.assertEqual(self.command_history, expected_cmds)

    def test_quaternion_setup(self):
        expected_cmds= [
            [0x0b, 0x02, 0x19, 0x06, 0xff, 0x60],
            [0x0b, 0x02, 0x19, 0x06, 0xff, 0x64],
            [0x0b, 0x02, 0x19, 0x06, 0xff, 0x68],
            [0x0b, 0x02, 0x19, 0x06, 0xff, 0x0c]
        ]

        signal = self.libmetawear.mbl_mw_sensor_fusion_get_data_signal(self.board, SensorFusion.DATA_CORRECTED_MAG)
        self.libmetawear.mbl_mw_datasignal_log(signal, self.logger_ready)

        self.assertEqual(self.command_history, expected_cmds)

class TestLoggerSetup(TestMetaWearBase):
    def logger_ready_handler(self, logger):
        self.libmetawear.mbl_mw_logger_remove(logger)
        self.assertEqual(self.command_history, self.expected_cmds)

    def test_accelerometer(self):
        self.expected_cmds= [
            [0x0b, 0x02, 0x03, 0x04, 0xff, 0x60],
            [0x0b, 0x02, 0x03, 0x04, 0xff, 0x24],
            [0x0b, 0x03, 0x00],
            [0x0b, 0x03, 0x01]
        ]

        logger_ready= Fn_VoidPtr(self.logger_ready_handler)
        self.test_signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_log(self.test_signal, logger_ready)

class TestLoggerTimeout(TestMetaWearBase):
    def logger_ready(self, logger):
        self.created_logger= logger

    def logger_ready_snd(self, logger):
        self.e.set()

    def commandLogger(self, board, characteristic, command, length):
        if (command[0] == 0xb and command[1] == 0x2):
            response= create_string_buffer(b'\x0b\x00', 2)
            self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response.raw))
        else:
            super().commandLogger(board, characteristic, command, length)

    def test_timeout(self):
        self.e= threading.Event()

        logger_ready_fn= Fn_VoidPtr(self.logger_ready)
        logger_ready_snd_fn= Fn_VoidPtr(self.logger_ready_snd)
        self.test_signal= self.libmetawear.mbl_mw_acc_get_acceleration_data_signal(self.board)
        self.libmetawear.mbl_mw_datasignal_log(self.test_signal, logger_ready_fn)
        self.libmetawear.mbl_mw_datasignal_log(self.test_signal, logger_ready_fn)
        self.libmetawear.mbl_mw_datasignal_log(self.test_signal, logger_ready_snd_fn)

        self.e.wait()
        self.assertIsNone(self.created_logger)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`accelerometer`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-10

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time
from ctypes import cast, POINTER, c_float

from pymetawear.discover import select_device
from pymetawear.client import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear.core import CartesianFloat, DataTypeId, Fn_DataPtr
from pymetawear.client import MetaWearClient


address = select_device()
c = MetaWearClient(str(address), 'pygatt', debug=True)
print("New client created: {0}".format(c))


def acc_callback(data):
    """Handle a (x,y,z) accelerometer tuple."""
    if data.contents.type_id == DataTypeId.CARTESIAN_FLOAT:
        data_ptr = cast(data.contents.value, POINTER(CartesianFloat))
        print("X: {0}, Y: {1}, Z: {2}".format(*(data_ptr.contents.x,
                                     data_ptr.contents.y,
                                     data_ptr.contents.z)))
    else:
        raise PyMetaWearException('Incorrect data type id: ' +
            str(data.contents.type_id))
_acc_callback = Fn_DataPtr(acc_callback)


print("Write accelerometer settings...")
libmetawear.mbl_mw_acc_set_odr(c.board, c_float(50.0))
libmetawear.mbl_mw_acc_set_range(c.board, c_float(4.0))
print("Write accelerometer config....")
libmetawear.mbl_mw_acc_write_acceleration_config(c.board)

print("Subscribing to accelerometer signal...")
data_signal = libmetawear.mbl_mw_acc_get_acceleration_data_signal(c.board)
libmetawear.mbl_mw_datasignal_subscribe(data_signal, _acc_callback)
print("Enable acc sampling on board...")
libmetawear.mbl_mw_acc_enable_acceleration_sampling(c.board)
print("Start acc on board...")
libmetawear.mbl_mw_acc_start(c.board)

time.sleep(10.0)


print("Stop accelerometer...")
libmetawear.mbl_mw_acc_stop(c.board)
print("Disable accelerometer sampling...")
libmetawear.mbl_mw_acc_disable_acceleration_sampling(c.board)
print("Unsubscribe to notification...")
libmetawear.mbl_mw_datasignal_unsubscribe(data_signal)

time.sleep(5.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`battery`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()
c = MetaWearClient(str(address), 'pygatt', timeout=10, debug=True)
print("New client created: {0}".format(c))
c.connect()


def temperature_callback(data):
    epoch = data[0]
    temp = data[1]
    print("[{0}] Temperature {1} \u2103".format(epoch, temp))

for channel in c.temperature.channels:

    c.temperature.set_settings(channel)
    time.sleep(0.1)

    print("Subscribe to {0} Temperature notifications...".format(channel))
    c.temperature.notifications(temperature_callback)
    time.sleep(1.0)

    print("Trigger {0} temperature notification...".format(channel))
    c.temperature.read_temperature()
    time.sleep(1.0)

    print("Unsubscribe to temperature notifications...")
    c.temperature.notifications(None)
    time.sleep(1.0)

c.disconnect()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: jboeer <jonas.boeer@kinemic.de>

Created: 2016-04-14

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import re
import logging
from functools import wraps
from ctypes import cast, POINTER

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear import sensor
from pymetawear.mbientlab.metawear.core import DataTypeId, CartesianFloat
from pymetawear.modules.base import PyMetaWearModule, Modules

log = logging.getLogger(__name__)


def require_bmm150(f):
    def wrapper(*args, **kwargs):
        if getattr(args[0], 'mag_class', None) is None:
            raise PyMetaWearException("There is not Magnetometer "
                                      "module on your MetaWear board!")
        return f(*args, **kwargs)

    return wrapper


class MagnetometerModule(PyMetaWearModule):
    """MetaWear accelerometer module implementation.

    :param ctypes.c_long board: The MetaWear board pointer value.
    :param int module_id: The module id of this accelerometer
        component, obtained from ``libmetawear``.
    :param bool debug: If ``True``, module prints out debug information.

    """

    def __init__(self, board, module_id, debug=False):
        super(MagnetometerModule, self).__init__(board, debug)
        self.module_id = module_id

        if self.module_id == Modules.MBL_MW_MODULE_NA:
            # No magnetometer present!
            self.mag_class = None
        else:
            self.mag_class = sensor.MagnetometerBmm150

        if self.mag_class is not None:
            self.power_presets = {re.search('^PRESET_([A-Z\_]*)', k).groups()[0].lower():
                                  getattr(self.mag_class, k, None) for k in filter(
                lambda x: x.startswith('PRESET_'), vars(self.mag_class))}

        if debug:
            log.setLevel(logging.DEBUG)

    def __str__(self):
        return "{0} {1}: Power presets: {2}".format(
            self.module_name, self.sensor_name,
            [float(k) for k in sorted(self.power_presets.keys())])

    def __repr__(self):
        return str(self)

    @property
    def module_name(self):
        return "Magnetometer"

    @property
    def sensor_name(self):
        if self.mag_class is not None:
            return self.mag_class.__name__.replace('Magnetometer', '')
        else:
            return ''

    @property
    @require_bmm150
    def data_signal(self):
        return libmetawear.mbl_mw_mag_bmm150_get_b_field_data_signal(self.board)

    def _get_power_preset(self, value):
        if value.lower() in self.power_presets:
            return self.power_presets.get(value.lower())
        else:
            raise ValueError("Requested power preset ({0}) was not part of "
                             "possible values: {1}".format(value.lower(), [self.power_presets.keys()]))

    @require_bmm150
    def get_current_settings(self):
        raise NotImplementedError()

    @require_bmm150
    def get_possible_settings(self):
        return {
            'power_preset': [sorted(self.power_presets.keys(), key=lambda x:(x.lower()))]
        }

    @require_bmm150
    def set_settings(self, power_preset):
        """Set magnetometer settings.

         .code-block:: python

            mwclient.magnetometer.set_settings(power_preset="LOW_POWER")

        Call :meth:`~get_possible_settings` to see which values
        that can be set for this sensor.

        :param str power_preset: The power preset, influencing the data rate,
            accuracy and power consumption

        """
        pp = self._get_power_preset(power_preset)
        if self._debug:
            log.debug("Setting Magnetometer power preset to {0}".format(pp))
        libmetawear.mbl_mw_mag_bmm150_set_preset(self.board, pp)

    @require_bmm150
    def notifications(self, callback=None):
        """Subscribe or unsubscribe to magnetometer notifications.

        Convenience method for handling magnetometer usage.

        Example:

        .. code-block:: python

            def handle_notification(data):
                # Handle a (epoch_time, (x,y,z)) magnetometer tuple.
                epoch = data[0]
                xyz = data[1]
                print("[{0}] X: {1}, Y: {2}, Z: {3}".format(epoch, *xyz))

            mwclient.magnetometer.notifications(handle_notification)

        :param callable callback: Magnetometer notification callback function.
            If `None`, unsubscription to magnetometer notifications is registered.

        """
        if callback is None:
            self.stop()
            self.toggle_sampling(False)
            super(MagnetometerModule, self).notifications(None)
        else:
            super(MagnetometerModule, self).notifications(
                sensor_data(callback))
            self.toggle_sampling(True)
            self.start()

    @require_bmm150
    def start(self):
        """Switches the magnetometer to active mode."""
        libmetawear.mbl_mw_mag_bmm150_start(self.board)

    @require_bmm150
    def stop(self):
        """Switches the magnetometer to standby mode."""
        libmetawear.mbl_mw_mag_bmm150_stop(self.board)

    @require_bmm150
    def toggle_sampling(self, enabled=True):
        """Enables or disables magnetometer sampling.

        :param bool enabled: Desired state of the magnetometer.

        """
        if enabled:
            libmetawear.mbl_mw_mag_bmm150_enable_b_field_sampling(self.board)
        else:
            libmetawear.mbl_mw_mag_bmm150_disable_b_field_sampling(self.board)


def sensor_data(func):
    @wraps(func)
    def wrapper(data):
        if data.contents.type_id == DataTypeId.CARTESIAN_FLOAT:
            epoch = int(data.contents.epoch)
            data_ptr = cast(data.contents.value, POINTER(CartesianFloat))
            func((epoch, (data_ptr.contents.x,
                          data_ptr.contents.y,
                          data_ptr.contents.z)))
        else:
            raise PyMetaWearException('Incorrect data type id: {0}'.format(
                data.contents.type_id))

    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-08

"""
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import os
import time
from ctypes import byref
import uuid
import logging

from pymetawear import libmetawear
from pymetawear.exceptions import PyMetaWearException
from pymetawear.mbientlab.metawear.core import BtleConnection, \
    Fn_VoidPtr_GattCharPtr, \
    Fn_VoidPtr_GattCharPtr_ByteArray, Fn_VoidPtr_Int
from pymetawear.specs import METAWEAR_SERVICE_NOTIFY_CHAR
from pymetawear.compat import string_types, range_

log = logging.getLogger(__name__)


class BLECommunicationBackend(object):
    def __init__(self, address, interface=None, timeout=None, debug=False):
        self._address = str(address)
        self._interface = str(interface)
        self._debug = debug
        self._timeout = timeout

        if debug:
            log.setLevel(logging.DEBUG)

        self.initialization_status = -1

        self._requester = None

        # Define read and write to characteristics methods to be used by
        # libmetawear. These methods in their turn use the backend read/write
        # methods implemented in the specific backends.
        self._btle_connection = BtleConnection(
            write_gatt_char=Fn_VoidPtr_GattCharPtr_ByteArray(
                self.mbl_mw_write_gatt_char),
            read_gatt_char=Fn_VoidPtr_GattCharPtr(self.mbl_mw_read_gatt_char))

        # Dictionary of callbacks for subscriptions set up through the
        # libmetawear library.
        self.callbacks = {
            'initialization': (self._initialized_fcn,
                               Fn_VoidPtr_Int(self._initialized_fcn)),
        }

        self.board = None
        self._notify_char_handle = None

    def __str__(self):
        return "{0}, {1}".format(self.__class__.__name__, self._address)

    def __repr__(self):
        return "<{0}, {1}>".format(self.__class__.__name__, self._address)

    def _build_handle_dict(self):
        pass

    @property
    def initialized(self):
        return self.initialization_status >= 0

    @property
    def is_connected(self):
        raise NotImplementedError("Use backend-specific classes instead!")

    @property
    def requester(self):
        """The requester object for the backend used.

        :return: The connected requester instance.
        :rtype: :class:`bluetooth.ble.GATTRequester` or
            :class:`pygatt.device.BLEDevice`

        """
        raise NotImplementedError("Use backend-specific classes instead!")

    def connect(self, clean_connect=False):
        self._build_handle_dict()

        # Setup the notification characteristic subscription
        # required by MetaWear.
        self._notify_char_handle = self.get_handle(
            METAWEAR_SERVICE_NOTIFY_CHAR[1])
        self.subscribe(METAWEAR_SERVICE_NOTIFY_CHAR[1],
                       self.handle_notify_char_output)

        # Now create a libmetawear board object and initialize it.
        # Free memory for any old board first.
        if self.board is not None:
            try:
                libmetawear.mbl_mw_metawearboard_tear_down(self.board)
            except:
                pass
            libmetawear.mbl_mw_metawearboard_free(self.board)
        self.board = libmetawear.mbl_mw_metawearboard_create(
            byref(self._btle_connection))

        _response_time = os.environ.get('PYMETAWEAR_RESPONSE_TIME', 300)
        libmetawear.mbl_mw_metawearboard_set_time_for_response(self.board, int(
            _response_time))

        libmetawear.mbl_mw_metawearboard_initialize(
            self.board, self.callbacks.get('initialization')[1])

    def disconnect(self):
        """Handle any required disconnecting in the backend,
        e.g. sever Bluetooth connection.
        """
        raise NotImplementedError("Use backend-specific classes instead!")

    def subscribe(self, characteristic_uuid, callback):
        self._subscribe(characteristic_uuid, callback)
        self._log("Subscribe", characteristic_uuid, [], 0)

    def mbl_mw_read_gatt_char(self, board, characteristic):
        """Read the desired data from the MetaWear board.

        :param pymetawear.mbientlab.metawear.core.GattCharacteristic
            characteristic: :class:`ctypes.POINTER` to a GattCharacteristic.

        """
        response = self.read_gatt_char_by_uuid(characteristic)
        sb = self._response_2_string_buffer(response)
        libmetawear.mbl_mw_metawearboard_char_read(
            self.board, characteristic, sb.raw, len(sb.raw))

        self._log("Read", characteristic, response, len(response))

    def mbl_mw_write_gatt_char(self, board, characteristic, command, length):
        """Write the desired data to the MetaWear board.

        :param pymetawear.mbientlab.metawear.core.GattCharacteristic
            characteristic: Characteristic to write to.
        :param POINTER command: The command to send, as a byte array pointer.
        :param int length: Length of the array that command points.

        """
        self._log("Write", characteristic, command, length)
        self.write_gatt_char_by_uuid(characteristic, command, length)

    # Helper methods

    @staticmethod
    def get_uuid(characteristic):
        if isinstance(characteristic, uuid.UUID):
            return characteristic
        else:
            return uuid.UUID(int=(characteristic.contents.uuid_high << 64) +
                             characteristic.contents.uuid_low)

    @staticmethod
    def get_service_uuid(characteristic):
        if isinstance(characteristic, uuid.UUID):
            return characteristic
        else:
            return uuid.UUID(int=(characteristic.contents.service_uuid_high << 64) +
                             characteristic.contents.service_uuid_low)

    def sleep(self, t):
        """Make backend sleep."""
        time.sleep(t)

    #  Callback methods

    def _initialized_fcn(self, board, status):
        log.debug("{0} initialized with status {1}.".format(self, status))
        self.initialization_status = status

    def handle_notify_char_output(self, handle, value):

        self._log("Notify", handle, value, 0)

        if handle == self._notify_char_handle:
            sb = self._response_2_string_buffer(value)
            libmetawear.mbl_mw_metawearboard_notify_char_changed(
                self.board, sb.raw, len(sb.raw))
        else:
            raise PyMetaWearException(
                "Notification on unexpected handle: {0}".format(handle))

    # Methods to be implemented by backends.

    def _subscribe(self, characterisitic_uuid, callback):
        raise NotImplementedError("Use backend-specific classes instead!")

    def read_gatt_char_by_uuid(self, characteristic):
        raise NotImplementedError("Use backend-specific classes instead!")

    def write_gatt_char_by_uuid(self, characteristic, command, length):
        raise NotImplementedError("Use backend-specific classes instead!")

    def get_handle(self, uuid, notify_handle=False):
        """Get handle for a characteristic UUID.

        :param uuid.UUID uuid: The UUID to get handle of.
        :param bool notify_handle:
        :return: Integer handle corresponding to the input characteristic UUID.
        :rtype: int

        """
        raise NotImplementedError("Use backend-specific classes instead!")

    def _response_2_string_buffer(self, response):
        raise NotImplementedError("Use backend-specific classes instead!")

    # Debug method

    def _log(self, action, handle_or_char, value, value_length):

        if not self._debug:
            return

        if action == "Write":
            data_as_hex = " ".join(["{:02x}".format(b) for
                                    b in [value[i] for i in range_(value_length)]])
        elif action == "Subscribe":
            data_as_hex = ""
        else:
            data_as_hex = " ".join(["{:02x}".format(ord(b)) for
                                    b in self._response_2_string_buffer(value)])

        if isinstance(handle_or_char, (uuid.UUID, string_types)):
            handle = self.get_handle(handle_or_char)
        elif isinstance(handle_or_char, int):
            handle = handle_or_char
        else:
            # Assume it is a Pointer to a GattCharacteristic...
            try:
                handle = self.get_handle(self.get_uuid(handle_or_char))
            except:
                handle = -1

        log.debug("{0:<6s} 0x{1:04x}: {2}".format(action, handle, data_as_hex))



from ctypes import byref, create_string_buffer
from common import TestMetaWearBase
from mbientlab.metawear.peripheral import Led

class TestLedControl(TestMetaWearBase):
    def test_play(self):
        expected= [0x02, 0x01, 0x01]

        self.libmetawear.mbl_mw_led_play(self.board)
        self.assertEqual(self.command, expected)

    def test_autoplay(self):
        expected= [0x02, 0x01, 0x02]

        self.libmetawear.mbl_mw_led_autoplay(self.board)
        self.assertEqual(self.command, expected)

    def test_pause(self):
        expected= [0x02, 0x01, 0x00]

        self.libmetawear.mbl_mw_led_pause(self.board)
        self.assertEqual(self.command, expected)

    def test_stop_clear(self):
        expected= [0x02, 0x02, 0x01]

        self.libmetawear.mbl_mw_led_stop_and_clear(self.board)
        self.assertEqual(self.command, expected)

    def test_stop_no_clear(self):
        expected= [0x02, 0x02, 0x00]

        self.libmetawear.mbl_mw_led_stop(self.board, 0)
        self.assertEqual(self.command, expected)

class TestLedPattern(TestMetaWearBase):
    def test_blink_pattern(self):
        expected= [0x02, 0x03, 0x00, 0x02, 0x1f, 0x00, 0x00, 0x00, 0x32, 0x00, 0x00, 0x00, 0xf4, 0x01, 0x00, 0x00, 0x0a]
        pattern= Led.Pattern(delay_time_ms= 5000, repeat_count= 10)

        self.libmetawear.mbl_mw_led_load_preset_pattern(byref(pattern), Led.PRESET_BLINK)
        self.libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), Led.COLOR_GREEN)

        self.assertEqual(self.command, expected)

    def test_solid_pattern(self):
        expected= [0x02, 0x03, 0x01, 0x02, 0x1f, 0x1f, 0x00, 0x00, 0xf4, 0x01, 0x00, 0x00, 0xE8, 0x03, 0x00, 0x00, 0x14]
        pattern= Led.Pattern(delay_time_ms= 10000, repeat_count= 20)

        self.libmetawear.mbl_mw_led_load_preset_pattern(byref(pattern), Led.PRESET_SOLID)
        self.libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), Led.COLOR_RED)

        self.assertEqual(self.command, expected)

    def test_pulse_pattern(self):
        expected= [0x02, 0x03, 0x02, 0x02, 0x1f, 0x00, 0xd5, 0x02, 0xf4, 0x01, 0xd5, 0x02, 0xd0, 0x07, 0x00, 0x00, 0x28]
        pattern= Led.Pattern(delay_time_ms= 12345, repeat_count= 40)

        self.libmetawear.mbl_mw_led_load_preset_pattern(byref(pattern), Led.PRESET_PULSE)
        self.libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), Led.COLOR_BLUE)

        self.assertEqual(self.command, expected)

class TestLedDelayedPattern(TestMetaWearBase):
    def setUp(self):
        self.metawear_r_services[0x2]= create_string_buffer(b'\x02\x80\x00\x01\x03\x00', 6)
        super().setUp()

    def test_blink_pattern(self):
        expected= [0x02, 0x03, 0x00, 0x02, 0x1f, 0x00, 0x00, 0x00, 0x32, 0x00, 0x00, 0x00, 0xf4, 0x01, 0x88, 0x13, 0x0a]
        pattern= Led.Pattern(delay_time_ms= 5000, repeat_count= 10)

        self.libmetawear.mbl_mw_led_load_preset_pattern(byref(pattern), Led.PRESET_BLINK)
        self.libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), Led.COLOR_GREEN)

        self.assertEqual(self.command, expected)

    def test_solid_pattern(self):
        expected= [0x02, 0x03, 0x01, 0x02, 0x1f, 0x1f, 0x00, 0x00, 0xf4, 0x01, 0x00, 0x00, 0xE8, 0x03, 0x10, 0x27, 0x14]
        pattern= Led.Pattern(delay_time_ms= 10000, repeat_count= 20)

        self.libmetawear.mbl_mw_led_load_preset_pattern(byref(pattern), Led.PRESET_SOLID)
        self.libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), Led.COLOR_RED)

        self.assertEqual(self.command, expected)

    def test_pulse_pattern(self):
        expected= [0x02, 0x03, 0x02, 0x02, 0x1f, 0x00, 0xd5, 0x02, 0xf4, 0x01, 0xd5, 0x02, 0xd0, 0x07, 0x39, 0x30, 0x28]
        pattern= Led.Pattern(delay_time_ms= 12345, repeat_count= 40)

        self.libmetawear.mbl_mw_led_load_preset_pattern(byref(pattern), Led.PRESET_PULSE)
        self.libmetawear.mbl_mw_led_write_pattern(self.board, byref(pattern), Led.COLOR_BLUE)

        self.assertEqual(self.command, expected)



from common import TestMetaWearBase
from mbientlab.metawear.core import CartesianFloat
from mbientlab.metawear.sensor import Accelerometer, AccelerometerBma255, AccelerometerBosch
from ctypes import create_string_buffer

class TestAccBma255Config(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_DETECT_BOARD

        super().setUp()

    def test_set_range(self):
        tests= [
            {
                'expected': [0x03, 0x03, 0x0a, 0x03],
                'range': AccelerometerBosch.FSR_2G,
                'range_name': '2g'
            },
            {
                'expected': [0x03, 0x03, 0x0a, 0x05],
                'range': AccelerometerBosch.FSR_4G,
                'range_name': '4g'
            },
            {
                'expected': [0x03, 0x03, 0x0a, 0x08],
                'range': AccelerometerBosch.FSR_8G,
                'range_name': '8g'
            },
            {
                'expected': [0x03, 0x03, 0x0a, 0x0c],
                'range': AccelerometerBosch.FSR_16G,
                'range_name': '16g'
            }
        ]

        for test in tests:
            with self.subTest(range= test['range_name']):
                self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, test['range'])
                self.libmetawear.mbl_mw_acc_bosch_write_acceleration_config(self.board)
                self.assertEqual(self.command, test['expected'])

    def test_set_odr(self):
        tests= [
            {
                'expected': [0x03, 0x03, 0x08, 0x03],
                'odr': AccelerometerBma255.ODR_15_62HZ,
                'odr_name': '15.62Hz'
            },
            {
                'expected': [0x03, 0x03, 0x09, 0x03],
                'odr': AccelerometerBma255.ODR_31_26HZ,
                'odr_name': '31.26Hz'
            },
            {
                'expected': [0x03, 0x03, 0x0a, 0x03],
                'odr': AccelerometerBma255.ODR_62_5HZ,
                'odr_name': '62.5Hz'
            },
            {
                'expected': [0x03, 0x03, 0x0b, 0x03],
                'odr': AccelerometerBma255.ODR_125HZ,
                'odr_name': '125Hz'
            },
            {
                'expected': [0x03, 0x03, 0x0c, 0x03],
                'odr': AccelerometerBma255.ODR_250HZ,
                'odr_name': '250Hz'
            },
            {
                'expected': [0x03, 0x03, 0x0d, 0x03],
                'odr': AccelerometerBma255.ODR_500HZ,
                'odr_name': '500Hz'
            },
            {
                'expected': [0x03, 0x03, 0x0e, 0x03],
                'odr': AccelerometerBma255.ODR_1000HZ,
                'odr_name': '1000Hz'
            },
            {
                'expected': [0x03, 0x03, 0x0f, 0x03],
                'odr': AccelerometerBma255.ODR_2000HZ,
                'odr_name': '2000Hz'
            }
        ]

        for test in tests:
            with self.subTest(odr= test['odr_name']):
                self.libmetawear.mbl_mw_acc_bma255_set_odr(self.board, test['odr'])
                self.libmetawear.mbl_mw_acc_bosch_write_acceleration_config(self.board)
                self.assertEqual(self.command, test['expected'])

    def test_set_odr_and_range(self):
        expected= [0x03, 0x03, 0x0c, 0x05]

        self.libmetawear.mbl_mw_acc_bma255_set_odr(self.board, AccelerometerBma255.ODR_250HZ)
        self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccelerometerBosch.FSR_4G)
        self.libmetawear.mbl_mw_acc_bosch_write_acceleration_config(self.board)
        self.assertListEqual(self.command, expected)

    def test_enable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x01, 0x00]

        self.libmetawear.mbl_mw_acc_bosch_enable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

    def test_disable_acceleration_sampling(self):
        expected= [0x03, 0x02, 0x00, 0x01]

        self.libmetawear.mbl_mw_acc_bosch_disable_acceleration_sampling(self.board)
        self.assertListEqual(self.command, expected)

class TestAccBma255Data(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_DETECT_BOARD

        super().setUp()

        self.signal= self.libmetawear.mbl_mw_acc_bosch_get_acceleration_data_signal(self.board)

    def test_subscribe_data(self):
        expected= [0x03, 0x04, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.signal, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_unsubscribe_data(self):
        expected= [0x03, 0x04, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.signal)
        self.assertListEqual(self.command, expected)

    def test_handle_data(self):
        response= create_string_buffer(b'\x03\x04\xe1\xb3\xa1\x24\xb1\x2e', 8)
        expected= CartesianFloat(x= -4.7576, y= 2.2893, z= 2.9182)

        self.libmetawear.mbl_mw_datasignal_subscribe(self.signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccelerometerBosch.FSR_8G)
        status= self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))
        self.assertEqual(self.data_cartesian_float, expected)

    def test_handle_data_component(self):
        response= create_string_buffer(b'\x03\x04\xe1\xb3\xa1\x24\xb1\x2e', 8)
        tests= [
            {
                'expected': -4.7576,
                'index': Accelerometer.ACCEL_X_AXIS_INDEX,
                'name': 'x-axis'
            },
            {
                'expected': 2.2893,
                'index': Accelerometer.ACCEL_Y_AXIS_INDEX,
                'name': 'y-axis'
            },
            {
                'expected': 2.9182,
                'index': Accelerometer.ACCEL_Z_AXIS_INDEX,
                'name': 'z-axis'
            }
        ]

        for test in tests:
            with self.subTest(odr= test['name']):
                acc_component = self.libmetawear.mbl_mw_datasignal_get_component(self.signal, test['index'])
                self.libmetawear.mbl_mw_datasignal_subscribe(acc_component, self.sensor_data_handler)
                self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccelerometerBosch.FSR_8G)
                self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))

                self.assertAlmostEqual(self.data_float.value, test['expected'], delta = 0.0001)

class TestBma255HighFreqAccdata(TestMetaWearBase):
    def setUp(self):
        self.boardType= TestMetaWearBase.METAWEAR_ENV_BOARD

        super().setUp()

        self.accel_data_signal= self.libmetawear.mbl_mw_acc_bosch_get_high_freq_acceleration_data_signal(self.board)
        
    def sensorDataHandler(self, data):
        super().sensorDataHandler(data)

        self.cartesian_float_values.append(self.data_cartesian_float)

    def test_handle_packed_data(self):
        response = create_string_buffer(b'\x03\x1c\x31\x26\x55\xf9\x65\x77\x39\x29\x89\xdb\xfd\x7f\x95\x3d\x61\xc1\xf1\x7f', 20)
        expected_values = [CartesianFloat(x = 1.1935, y = -0.2084, z = 3.7311), CartesianFloat(x = 1.2882, y = -1.1395, z = 3.9996), CartesianFloat(x = 1.9244, y = -1.9569, z = 3.9982)]

        self.cartesian_float_values= []
        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, self.sensor_data_handler)
        self.libmetawear.mbl_mw_acc_bosch_set_range(self.board, AccelerometerBosch.FSR_4G)
        self.libmetawear.mbl_mw_connection_notify_char_changed(self.board, response.raw, len(response))
        self.assertEqual(self.cartesian_float_values, expected_values)

    def test_subscribe(self):
        expected= [0x03, 0x1c, 0x01]

        self.libmetawear.mbl_mw_datasignal_subscribe(self.accel_data_signal, self.sensor_data_handler)
        self.assertListEqual(self.command, expected)

    def test_unsubscribe(self):
        expected= [0x03, 0x1c, 0x00]

        self.libmetawear.mbl_mw_datasignal_unsubscribe(self.accel_data_signal)
        self.assertListEqual(self.command, expected)



from common import TestMetaWearBase

class TestDebug(TestMetaWearBase):
    def test_mbl_mw_debug_reset(self):
        expected= [0xfe, 0x01]

        self.libmetawear.mbl_mw_debug_reset(self.board)
        self.assertListEqual(self.command, expected)

    def test_mbl_mw_debug_jump_to_bootloader(self):
        expected= [0xfe, 0x02]

        self.libmetawear.mbl_mw_debug_jump_to_bootloader(self.board)
        self.assertEqual(self.command, expected)

    def test_mbl_mw_debug_disconnect(self):
        expected= [0xfe, 0x06]

        self.libmetawear.mbl_mw_debug_disconnect(self.board)
        self.assertEqual(self.command, expected)

    def bml_mw_debug_reset_after_gc(self):
        expected= [0xfe, 0x05]

        self.libmetawear.mbl_mw_debug_reset_after_gc(self.board)
        self.assertEqual(self.command, expected)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`battery`
==================

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2016-04-02

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import time

from pymetawear.discover import select_device
from pymetawear.client import MetaWearClient

address = select_device()
c = MetaWearClient(str(address), 'pygatt', debug=True)
print("New client created: {0}".format(c))


def battery_callback(data):
    """Handle a battery status tuple."""
    epoch = data[0]
    battery = data[1]
    print("[{0}] Voltage: {1}, Charge: {2}".format(
        epoch, battery[0], battery[1]))


print("Subscribe to battery notifications...")
c.battery.notifications(battery_callback)
time.sleep(1.0)

print("Trigger battery state notification...")
c.battery.read_battery_state()
time.sleep(1.0)

print("Unsubscribe to battery notifications...")
c.battery.notifications(None)
time.sleep(1.0)

c.disconnect()
